@@ -14,34 +14,50 @@ specific language governing permissions and limitations
 under the License.
 """
 import datetime as dt
+
+import backoff
+import deprecation
 import logging
+from time import sleep
 from typing import Tuple, Union, List, Dict
 
+from gs_quant.api.api_session import ApiWithCustomSession
 from gs_quant.common import PositionType
 from gs_quant.common import RiskRequest, Currency
+from gs_quant.errors import MqInternalServerError, MqTimeoutError, MqRateLimitedError
 from gs_quant.instrument import Instrument
 from gs_quant.session import GsSession
-from gs_quant.target.portfolios import Portfolio, Position, PositionSet
+from gs_quant.target.portfolios import Portfolio, Position, PositionSet, PortfolioTree
 from gs_quant.target.reports import Report
 from gs_quant.target.risk_models import RiskModelTerm as Term
-from gs_quant.target.workflow_quote import WorkflowPosition, WorkflowPositionsResponse, SaveQuoteRequest
+from gs_quant.workflow import WorkflowPosition, WorkflowPositionsResponse, SaveQuoteRequest
 
 _logger = logging.getLogger(__name__)
 
 
-class GsPortfolioApi:
-    """GS Asset API client implementation"""
+class GsPortfolioApi(ApiWithCustomSession):
+    """GS Asset API client implementation.
+
+    To pass additional query parameters, use kwargs."""
 
     @classmethod
     def get_portfolios(cls,
                        portfolio_ids: List[str] = None,
                        portfolio_names: List[str] = None,
-                       limit: int = 100) -> Tuple[Portfolio, ...]:
+                       limit: int = 100,
+                       **kwargs) -> Tuple[Portfolio, ...]:
         url = '/portfolios?'
         if portfolio_ids:
             url += f'&id={"&id=".join(portfolio_ids)}'
         if portfolio_names:
             url += f'&name={"&name=".join(portfolio_names)}'
+        for k, v in kwargs.items():
+            if isinstance(v, list):
+                for i in v:
+                    # in case v is not a list of strings
+                    url += f'&{k}={i}'
+            else:
+                url += f'&{k}={v}'
         return GsSession.current._get(f'{url}&limit={limit}', cls=Portfolio)['results']
 
     @classmethod
@@ -50,7 +66,7 @@ class GsPortfolioApi:
 
     @classmethod
     def get_portfolio_by_name(cls, name: str) -> Portfolio:
-        ret = GsSession.current._get('/portfolios?name={}'.format(name))
+        ret = cls.get_session()._get('/portfolios?name={}'.format(name))
         num_found = ret.get('totalResults', 0)
 
         if num_found == 0:
@@ -73,8 +89,19 @@ class GsPortfolioApi:
         return GsSession.current._delete('/portfolios/{id}'.format(id=portfolio_id))
 
     # manage portfolio positions
+    @classmethod
+    def get_portfolio_analyze(cls, portfolio_id: str) -> dict:
+        url = '/portfolios/{id}/analyze'.format(id=portfolio_id)
+        res = GsSession.current._get(url)
+        return res
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def get_positions(cls, portfolio_id: str, start_date: dt.date = None, end_date: dt.date = None,
                       position_type: str = 'close') -> Tuple[PositionSet, ...]:
         url = '/portfolios/{id}/positions?type={positionType}'.format(id=portfolio_id, positionType=position_type)
@@ -103,7 +130,7 @@ class GsPortfolioApi:
         else:
             url = '/risk-internal/{}/{}/positions'.format(root, positions_id)
 
-        results = GsSession.current._get(url, timeout=181)
+        results = cls.get_session()._get(url, timeout=181)
         return tuple(cls._unpack_position_set(res) for res in results['positionSets'])
 
     @classmethod
@@ -152,7 +179,7 @@ class GsPortfolioApi:
                                        prefer_instruments: bool = False) -> Tuple[Instrument, ...]:
         root = 'quote'
         url = '/risk{}/{}/{}'.format('-internal' if not prefer_instruments else '', root, workflow_id)
-        results = GsSession.current._get(url, timeout=181)
+        results = cls.get_session()._get(url, timeout=181)
 
         instruments = []
         for position in results.get('workflowPositions').get(workflow_id):
@@ -169,10 +196,16 @@ class GsPortfolioApi:
 
     @classmethod
     def get_position_dates(cls, portfolio_id: str) -> Tuple[dt.date, ...]:
-        position_dates = GsSession.current._get('/portfolios/{id}/positions/dates'.format(id=portfolio_id))['results']
+        position_dates = cls.get_session()._get('/portfolios/{id}/positions/dates'.format(id=portfolio_id))['results']
         return tuple(dt.datetime.strptime(d, '%Y-%m-%d').date() for d in position_dates)
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def update_positions(cls,
                          portfolio_id: str,
                          position_sets: List[PositionSet],
@@ -181,11 +214,18 @@ class GsPortfolioApi:
         return GsSession.current._put(url, position_sets)
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def get_positions_data(cls,
                            portfolio_id: str,
                            start_date: dt.date,
                            end_date: dt.date,
                            fields: List[str] = None,
+                           performance_report_id: str = None,
                            position_type: PositionType = None,
                            include_all_business_days: bool = False) -> List[dict]:
         start_date_str = start_date.isoformat()
@@ -193,6 +233,8 @@ class GsPortfolioApi:
         url = f'/portfolios/{portfolio_id}/positions/data?startDate={start_date_str}&endDate={end_date_str}'
         if fields is not None:
             url += '&fields='.join([''] + fields)
+        if performance_report_id is not None:
+            url += f'&reportId={performance_report_id}'
         if position_type is not None:
             url += '&type=' + position_type.value
         if include_all_business_days:
@@ -202,34 +244,34 @@ class GsPortfolioApi:
 
     @classmethod
     def update_quote(cls, quote_id: str, request: RiskRequest):
-        return GsSession.current._put('/risk-internal/quote/save/{id}'.format(id=quote_id), request)
+        return cls.get_session()._put('/risk-internal/quote/save/{id}'.format(id=quote_id), request)
 
     @classmethod
     def save_quote(cls, request: RiskRequest) -> str:
-        return GsSession.current._post('/risk-internal/quote/save', request)['results']
+        return cls.get_session()._post('/risk-internal/quote/save', request)['results']
 
     @classmethod
     def update_workflow_quote(cls, quote_id: str, request: SaveQuoteRequest):
         headers = {'Content-Type': 'application/x-msgpack'}
-        return GsSession.current._put('/risk-internal/quote/workflow/save/{id}'.format(id=quote_id), tuple([request]),
+        return cls.get_session()._put('/risk-internal/quote/workflow/save/{id}'.format(id=quote_id), tuple([request]),
                                       request_headers=headers)['results']
 
     @classmethod
     def save_workflow_quote(cls, request: SaveQuoteRequest) -> str:
         headers = {'Content-Type': 'application/x-msgpack'}
-        return GsSession.current._post('/risk-internal/quote/workflow/save', tuple([request]),
+        return cls.get_session()._post('/risk-internal/quote/workflow/save', tuple([request]),
                                        request_headers=headers)['results']
 
     @classmethod
     def share_workflow_quote(cls, request: SaveQuoteRequest) -> str:
         headers = {'Content-Type': 'application/x-msgpack'}
-        return GsSession.current._post('/risk-internal/quote/workflow/share', tuple([request]),
+        return cls.get_session()._post('/risk-internal/quote/workflow/share', tuple([request]),
                                        request_headers=headers)['results']
 
     @classmethod
-    def get_workflow_quote(cls, workflow_id: str) -> Tuple[WorkflowPosition]:
+    def get_workflow_quote(cls, workflow_id: str) -> Tuple[WorkflowPosition, ...]:
         url = f'/risk-internal/quote/workflow/{workflow_id}'
-        results = GsSession.current._get(url, timeout=181)
+        results = cls.get_session()._get(url, timeout=181)
         wf_pos_res = WorkflowPositionsResponse.from_dict(results)
         if wf_pos_res:
             return wf_pos_res.results
@@ -237,9 +279,9 @@ class GsPortfolioApi:
             return ()
 
     @classmethod
-    def get_shared_workflow_quote(cls, workflow_id: str) -> Tuple[WorkflowPosition]:
+    def get_shared_workflow_quote(cls, workflow_id: str) -> Tuple[WorkflowPosition, ...]:
         url = f'/risk-internal/quote/workflow/shared/{workflow_id}'
-        results = GsSession.current._get(url, timeout=181)
+        results = cls.get_session()._get(url, timeout=181)
         wf_pos_res = WorkflowPositionsResponse.from_dict(results)
         if wf_pos_res:
             return wf_pos_res.results
@@ -248,17 +290,33 @@ class GsPortfolioApi:
 
     @classmethod
     def save_to_shadowbook(cls, request: RiskRequest, name: str) -> str:
-        return GsSession.current._put(f'/risk-internal/shadowbook/save/{name}', request)['results']
+        return cls.get_session()._put(f'/risk-internal/shadowbook/save/{name}', request)['results']
 
     @classmethod
     def get_risk_models_by_coverage(cls, portfolio_id: str, term: Term = Term.Medium):
-        return GsSession.current._get(f'/portfolios/{portfolio_id}/models?sortByTerm={term.value}')['results']
+        return cls.get_session()._get(f'/portfolios/{portfolio_id}/models?sortByTerm={term.value}')['results']
 
     @classmethod
-    def get_reports(cls, portfolio_id: str) -> Tuple[Report, ...]:
-        return GsSession.current._get('/portfolios/{id}/reports'.format(id=portfolio_id), cls=Report)['results']
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
+    def get_reports(cls, portfolio_id: str, tags: Dict) -> Tuple[Report, ...]:
+        results = cls.get_session()._get('/portfolios/{id}/reports'.format(id=portfolio_id), cls=Report)['results']
+        if tags is not None:
+            tags_as_list = [{'name': key, 'value': tags[key]} for key in tags]
+            results = [r for r in results if r.parameters.tags == tags_as_list]
+        return results
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def schedule_reports(cls,
                          portfolio_id: str,
                          start_date: dt.date = None,
@@ -269,17 +327,37 @@ class GsPortfolioApi:
             payload['startDate'] = start_date.isoformat()
         if end_date is not None:
             payload['endDate'] = end_date.isoformat()
-        return GsSession.current._post('/portfolios/{id}/schedule'.format(id=portfolio_id), payload)
+        portfolio = cls.get_portfolio(portfolio_id)
+        if portfolio.tag_name_hierarchy is None or len(portfolio.tag_name_hierarchy) == 0:
+            cls.get_session()._post(f'/portfolios/{portfolio_id}/schedule', payload)
+        else:
+            count = 10
+            for report_id in portfolio.report_ids:
+                if count == 0:
+                    sleep(2)
+                    count = 10
+                else:
+                    cls.get_session()._post(f'/reports/{report_id}/schedule', payload)
+                    count -= 1
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def get_schedule_dates(cls,
                            portfolio_id: str,
                            backcast: bool = False) -> List[dt.date]:
-        results = GsSession.current._get(f'/portfolios/{portfolio_id}/schedule/dates?backcast={str(backcast).lower()}')
+        results = cls.get_session()._get(f'/portfolios/{portfolio_id}/schedule/dates?backcast={str(backcast).lower()}')
         return [dt.datetime.strptime(results['startDate'], '%Y-%m-%d').date(),
                 dt.datetime.strptime(results['endDate'], '%Y-%m-%d').date()]
 
     @classmethod
+    @deprecation.deprecated(deprecated_in='1.0.10',
+                            details='GsPortfolioApi.get_custom_aum is now deprecated, please use '
+                                    'GsReportApi.get_custom_aum instead.')
     def get_custom_aum(cls,
                        portfolio_id: str,
                        start_date: dt.date = None,
@@ -292,6 +370,9 @@ class GsPortfolioApi:
         return GsSession.current._get(url)['data']
 
     @classmethod
+    @deprecation.deprecated(deprecated_in='1.0.10',
+                            details='GsPortfolioApi.upload_custom_aum is now deprecated, please use '
+                                    'GsReportApi.upload_custom_aum instead.')
     def upload_custom_aum(cls,
                           portfolio_id: str,
                           aum_data: List[Dict],
@@ -302,17 +383,28 @@ class GsPortfolioApi:
             url += '?clearExistingData=true'
         return GsSession.current._post(url, payload)
 
+    @classmethod
+    def update_portfolio_tree(cls, portfolio_id: str):
+        return GsSession.current._post(f'/portfolios/{portfolio_id}/tree', {})
+
+    @classmethod
+    def get_portfolio_tree(cls, portfolio_id: str):
+        return GsSession.current._get(f'/portfolios/{portfolio_id}/tree', cls=PortfolioTree)
+
     @classmethod
     def get_attribution(cls,
                         portfolio_id: str,
                         start_date: dt.date = None,
                         end_date: dt.date = None,
-                        currency: Currency = None) -> Dict:
+                        currency: Currency = None,
+                        performance_report_id: str = None) -> Dict:
         url = f'/attribution/{portfolio_id}?'
         if start_date:
             url += f"&startDate={start_date.strftime('%Y-%m-%d')}"
         if end_date:
             url += f"&endDate={end_date.strftime('%Y-%m-%d')}"
         if currency:
-            url += f"currency={currency.value}"
+            url += f"&currency={currency.value}"
+        if performance_report_id:
+            url += f'&reportId={performance_report_id}'
         return GsSession.current._get(url)['results']
