@@ -18,9 +18,12 @@ import datetime as dt
 import logging
 from typing import Tuple, Dict, List, Union
 
+import backoff
+
+from gs_quant.errors import MqRateLimitedError, MqTimeoutError, MqInternalServerError
 from gs_quant.session import GsSession
 from gs_quant.target.risk_models import RiskModel, RiskModelCalendar, Factor, RiskModelData, \
-    RiskModelDataAssetsRequest, RiskModelDataMeasure, RiskModelEventType, RiskModelTerm
+    RiskModelDataAssetsRequest, RiskModelDataMeasure, RiskModelEventType, RiskModelTerm, IntradayFactorDataSource
 
 _logger = logging.getLogger(__name__)
 
@@ -33,6 +36,12 @@ class GsRiskModelApi:
         return GsSession.current._post('/risk/models', model, cls=RiskModel)
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def get_risk_model(cls, model_id: str) -> RiskModel:
         return GsSession.current._get(f'/risk/models/{model_id}', cls=RiskModel)
 
@@ -77,6 +86,12 @@ class GsRiskModelApi:
         return GsSession.current._delete(f'/risk/models/{model_id}')
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def get_risk_model_calendar(cls, model_id: str) -> RiskModelCalendar:
         return GsSession.current._get(f'/risk/models/{model_id}/calendar', cls=RiskModelCalendar)
 
@@ -85,6 +100,12 @@ class GsRiskModelApi:
         return GsSession.current._put(f'/risk/models/{model_id}/calendar', model_calendar, cls=RiskModelCalendar)
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def get_risk_model_dates(cls,
                              model_id: str,
                              start_date: dt.date = None,
@@ -128,12 +149,20 @@ class GsFactorRiskModelApi(GsRiskModelApi):
         return GsSession.current._delete(f'/risk/models/{model_id}/factors/{factor_id}')
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def get_risk_model_factor_data(cls,
                                    model_id: str,
                                    start_date: dt.date = None,
                                    end_date: dt.date = None,
                                    identifiers: List[str] = None,
-                                   include_performance_curve: bool = False) -> List[Dict]:
+                                   include_performance_curve: bool = False,
+                                   factor_categories: List[str] = None,
+                                   names: List[str] = None) -> List[Dict]:
         url = f'/risk/models/{model_id}/factors/data?'
         if start_date is not None:
             url += f'&startDate={start_date.strftime("%Y-%m-%d")}'
@@ -143,6 +172,42 @@ class GsFactorRiskModelApi(GsRiskModelApi):
             url += '&identifiers={ids}'.format(ids='&identifiers='.join(identifiers))
         if include_performance_curve:
             url += '&includePerformanceCurve=true'
+        if names:
+            url += '&name={names}'.format(names='&name='.join(names))
+        if factor_categories:
+            url += '&factorCategory={factor_categories}'\
+                .format(factor_categories='&factorCategory='.join(factor_categories))
+        return GsSession.current._get(url)['results']
+
+    @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
+    def get_risk_model_factor_data_intraday(cls,
+                                            model_id: str,
+                                            start_time: dt.datetime = None,
+                                            end_time: dt.datetime = None,
+                                            factor_ids: List[str] = None,
+                                            factor_categories: List[str] = None,
+                                            factors: List[str] = None,
+                                            data_source: Union[IntradayFactorDataSource, str] = None) -> List[Dict]:
+        url = f'/risk/models/{model_id}/factors/data/intraday?'
+        if start_time is not None:
+            url += f'&startTime={start_time.strftime("%Y-%m-%dT%H:%M:%SZ")}'
+        if end_time is not None:
+            url += f'&endTime={end_time.strftime("%Y-%m-%dT%H:%M:%SZ")}'
+        if factor_ids is not None:
+            url += '&factorId={ids}'.format(ids='&factorId='.join(factor_ids))
+        if data_source:
+            url += f'&dataSource={data_source if isinstance(data_source, str) else data_source.value}'
+        if factors:
+            url += '&factor={names}'.format(names='&factor='.join(factors))
+        if factor_categories:
+            url += '&factorCategory={factor_categories}' \
+                .format(factor_categories='&factorCategory='.join(factor_categories))
         return GsSession.current._get(url)['results']
 
     @classmethod
@@ -165,7 +230,8 @@ class GsFactorRiskModelApi(GsRiskModelApi):
                                model_data: Union[Dict, RiskModelData],
                                partial_upload: bool = False,
                                target_universe_size: float = None,
-                               final_upload: bool = None) -> str:
+                               final_upload: bool = None,
+                               aws_upload: bool = False) -> str:
         url = f'/risk/models/data/{model_id}'
         if partial_upload:
             url += '?partialUpload=true'
@@ -174,12 +240,23 @@ class GsFactorRiskModelApi(GsRiskModelApi):
             if final_upload is not None:
                 final_upload_flag = 'true' if final_upload else 'false'
                 url += f'&finalUpload={final_upload_flag}'
+            if aws_upload:
+                url += '&awsUpload=true'
+        else:
+            if aws_upload:
+                url += '?awsUpload=true'
         return GsSession.current._post(url, model_data, timeout=200)
 
     @classmethod
+    @backoff.on_exception(lambda: backoff.expo(base=2, factor=2),
+                          (MqTimeoutError, MqInternalServerError),
+                          max_tries=5)
+    @backoff.on_exception(lambda: backoff.constant(90),
+                          MqRateLimitedError,
+                          max_tries=5)
     def get_risk_model_data(cls, model_id: str, start_date: dt.date, end_date: dt.date = None,
                             assets: RiskModelDataAssetsRequest = None, measures: List[RiskModelDataMeasure] = None,
-                            limit_factors: bool = None) -> Dict:
+                            factors: list = None, limit_factors: bool = None) -> Dict:
         end_date = cls.get_risk_model_dates(model_id)[-1] if not end_date else end_date.strftime('%Y-%m-%d')
         query = {
             'startDate': start_date.strftime('%Y-%m-%d'),
@@ -189,6 +266,8 @@ class GsFactorRiskModelApi(GsRiskModelApi):
             query['assets'] = assets
         if measures is not None:
             query['measures'] = measures
+        if factors is not None:
+            query['factors'] = factors
         if limit_factors is not None:
             query['limitFactors'] = limit_factors
         return GsSession.current._post(f'/risk/models/data/{model_id}/query', query, timeout=200)
