@@ -17,22 +17,24 @@ import builtins
 import copy
 import datetime as dt
 import logging
+import sys
+import typing
 from abc import ABC, ABCMeta, abstractmethod
 from collections import namedtuple
 from dataclasses import Field, InitVar, MISSING, dataclass, field, fields, replace
-from enum import EnumMeta
+from enum import EnumMeta, Enum
 from functools import update_wrapper
 from typing import Iterable, Mapping, Optional, Union, Tuple
 
 import numpy as np
-from dataclasses_json import config, global_config
+from dataclasses_json import config, global_config, LetterCase, dataclass_json
 from dataclasses_json.core import _decode_generic, _is_supported_generic
 from inflection import camelize, underscore
 
 from gs_quant.context_base import ContextBase, ContextMeta
 from gs_quant.json_convertors import encode_date_or_str, decode_date_or_str, decode_optional_date, encode_datetime, \
     decode_datetime, decode_float_or_str, decode_instrument, encode_dictable, decode_quote_report, decode_quote_reports, \
-    decode_custom_comment, decode_custom_comments
+    decode_custom_comment, decode_custom_comments, decode_optional_time, encode_optional_time
 
 _logger = logging.getLogger(__name__)
 
@@ -41,12 +43,14 @@ __getattribute__ = object.__getattribute__
 __setattr__ = object.__setattr__
 
 _rename_cache = {}
+_is_supported_generic_cache = {}
+
 
 def exclude_none(o):
     return o is None
 
 
-def exlude_always(_o):
+def exclude_always(_o):
     return True
 
 
@@ -64,6 +68,16 @@ def _get_underscore(arg):
 
     return _rename_cache[arg]
 
+
+def _get_is_supported_generic(arg):
+    if arg in _is_supported_generic_cache:
+        is_supported_generic = _is_supported_generic_cache[arg]
+    else:
+        is_supported_generic = _is_supported_generic(arg)
+        _is_supported_generic_cache[arg] = is_supported_generic
+    return is_supported_generic
+
+
 def handle_camel_case_args(cls):
     init = cls.__init__
 
@@ -88,15 +102,31 @@ def handle_camel_case_args(cls):
     return cls
 
 
+def static_field(val):
+    return field(init=False, default=val)
+
+
 field_metadata = config(exclude=exclude_none)
-name_metadata = config(exclude=exlude_always)
+name_metadata = config(exclude=exclude_always)
 
 
 class RiskKey(namedtuple('RiskKey', ('provider', 'date', 'market', 'params', 'scenario', 'risk_measure'))):
 
     @property
     def ex_measure(self):
-        return RiskKey(self.provider, self.date, self.market, self.params, self.scenario, None)
+        from gs_quant.target.common import RiskRequestParameters  # noqa
+        return RiskKey(self.provider, self.date, self.market,
+                       RiskRequestParameters(self.params.csa_term, self.params.raw_results, False,
+                                             self.params.market_behaviour),
+                       self.scenario, None)
+
+    @property
+    def ex_historical_diddle(self):
+        from gs_quant.target.common import RiskRequestParameters  # noqa
+        return RiskKey(self.provider, self.date, self.market,
+                       RiskRequestParameters(self.params.csa_term, self.params.raw_results, False,
+                                             self.params.market_behaviour),
+                       self.scenario, self.risk_measure)
 
     @property
     def fields(self):
@@ -118,17 +148,29 @@ class EnumBase:
         return self.value < other.value
 
     def __repr__(self):
+        return str(self)
+
+    def __str__(self):
         return self.value
 
 
 class HashableDict(dict):
 
+    @staticmethod
+    def hashables(in_dict) -> Tuple:
+        hashables = []
+        for it in in_dict.items():
+            if isinstance(it[1], dict):
+                hashables.append((it[0], HashableDict.hashables(it[1])))
+            else:
+                hashables.append(it)
+        return tuple(hashables)
+
     def __hash__(self):
-        return hash(tuple(self.items()))
+        return hash(HashableDict.hashables(self))
 
 
 class DictBase(HashableDict):
-
     _PROPERTIES = set()
 
     def __init__(self, *args, **kwargs):
@@ -212,17 +254,48 @@ class Base(ABC):
 
         return super().__repr__()
 
+    @classmethod
+    def __is_type_match(cls, tp, val):
+        if sys.version_info >= (3, 9):
+            from types import GenericAlias
+            is_generic_alias = isinstance(tp, (typing._GenericAlias, GenericAlias))
+        else:
+            is_generic_alias = isinstance(tp, typing._GenericAlias)
+        if not is_generic_alias:
+            # Do not convert Enums to strings
+            is_enum_to_str = isinstance(val, Enum) and tp is str
+            return isinstance(tp, type) and (isinstance(val, tp) or is_enum_to_str)
+        if getattr(tp, '_special', False):
+            return False
+        origin = tp.__origin__
+        args = tp.__args__
+        if float in args:
+            args += (int,)
+        if origin == Union:
+            return any(cls.__is_type_match(arg, val) for arg in args)
+        if origin is tuple:
+            if not isinstance(val, tuple) or not args:
+                return False
+            if len(args) == 1 or args[1] == Ellipsis:
+                return all(cls.__is_type_match(args[0], x) for x in val)
+            else:
+                return len(args) == len(val) and all(cls.__is_type_match(arg, x) for arg, x in zip(args, val))
+        return False
+
     @classmethod
     def __coerce_value(cls, typ: type, value):
+        if cls.__is_type_match(typ, value):
+            return value
         if isinstance(value, np.generic):
             # Handle numpy types
             return value.item()
         elif hasattr(value, 'tolist'):
             # tolist converts scalar or array to native python type if not already native.
-            return value()
+            return value.tolist()
         elif typ in (DictBase, Optional[DictBase]) and isinstance(value, Base):
             return value.to_dict()
-        if _is_supported_generic(typ):
+        is_supported_generic = _get_is_supported_generic(typ)
+        if is_supported_generic:
             return _decode_generic(typ, value, False)
         else:
             return value
@@ -324,6 +397,7 @@ class Base(ABC):
                 __setattr__(self, fld.name, __getattribute__(instance, fld.name))
 
 
+@dataclass_json
 @dataclass
 class Priceable(Base):
 
@@ -461,12 +535,21 @@ class Scenario(Base, ContextBase, ABC, metaclass=__ScenarioMeta):
 
 @dataclass
 class RiskMeasureParameter(Base, ABC):
-    pass
+    def __repr__(self):
+        params = self.as_dict()
+        params.pop("parameter_type", None)
+        sorted_keys = sorted(params.keys(), key=lambda x: x.lower())
+        params = ", ".join(
+            [
+                f"{k}:{params[k].value if isinstance(params[k], EnumBase) else params[k]}"
+                for k in sorted_keys
+            ]
+        )
+        return f"{self.parameter_type}({params})"
 
 
 @dataclass
 class InstrumentBase(Base, ABC):
-
     quantity_: InitVar[float] = field(default=1, init=False)
 
     @property
@@ -587,6 +670,15 @@ def get_enum_value(enum_type: EnumMeta, value: Union[EnumBase, str]):
     return enum_value
 
 
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class MarketDataScenario(Base):
+    scenario: Scenario = field(default=None, metadata=field_metadata)
+    subtract_base: Optional[bool] = field(default=False, metadata=field_metadata)
+    name: Optional[str] = field(default=None, metadata=name_metadata)
+
+
 # Yes, I know this is a little evil ...
 global_config.encoders[dt.date] = dt.date.isoformat
 global_config.encoders[Optional[dt.date]] = encode_date_or_str
@@ -596,6 +688,10 @@ global_config.encoders[Union[dt.date, str]] = encode_date_or_str
 global_config.encoders[Optional[Union[dt.date, str]]] = encode_date_or_str
 global_config.decoders[Union[dt.date, str]] = decode_date_or_str
 global_config.decoders[Optional[Union[dt.date, str]]] = decode_date_or_str
+global_config.encoders[dt.time] = dt.time.isoformat
+global_config.decoders[dt.time] = dt.time.fromisoformat
+global_config.encoders[Optional[dt.time]] = encode_optional_time
+global_config.decoders[Optional[dt.time]] = decode_optional_time
 global_config.encoders[dt.datetime] = encode_datetime
 global_config.encoders[Optional[dt.datetime]] = encode_datetime
 global_config.decoders[dt.datetime] = decode_datetime
@@ -609,6 +705,5 @@ global_config.decoders[QuoteReport] = decode_quote_report
 global_config.decoders[Optional[Tuple[QuoteReport, ...]]] = decode_quote_reports
 global_config.decoders[CustomComments] = decode_custom_comment
 global_config.decoders[Optional[Tuple[CustomComments, ...]]] = decode_custom_comments
-
 global_config.encoders[Market] = encode_dictable
 global_config.encoders[Optional[Market]] = encode_dictable
