@@ -14,12 +14,15 @@ specific language governing permissions and limitations
 under the License.
 """
 
-import datetime as dt
+from gs_quant.target.common import *  # noqa
+from gs_quant.target.common import PayReceive as _PayReceive  # noqa
+from gs_quant.target.common import RiskMeasure as __RiskMeasure  # noqa
+from gs_quant.target.common import RiskMeasureType, AssetClass, RiskMeasureUnit  # noqa
+from gs_quant.base import EnumBase, RiskMeasureParameter  # noqa
 
-from gs_quant.target.common import *
-from gs_quant.target.common import PayReceive as _PayReceive
-from gs_quant.target.common import RiskMeasure as __RiskMeasure
-from gs_quant.target.common import RiskMeasureType, AssetClass
+from enum import Enum
+from typing import Union
+import datetime as dt
 
 
 class PositionType(Enum):
@@ -35,14 +38,12 @@ class PositionType(Enum):
 
 
 class DateLimit(Enum):
-    
     """ Datetime date constants """
 
     LOW_LIMIT = dt.date(1952, 1, 1)
 
 
 class PayReceive(EnumBase, Enum):
-
     """Pay or receive fixed"""
 
     Pay = 'Pay'
@@ -57,14 +58,6 @@ class PayReceive(EnumBase, Enum):
         return cls.Receive if key.lower() in ('receive', 'receiver') else super()._missing_(key)
 
 
-class MarketBehaviour(EnumBase, Enum):
-
-    """ContraintsBased or Calibrated"""
-
-    ContraintsBased = 'ContraintsBased'
-    Calibrated = 'Calibrated'
-
-
 class RiskMeasure(__RiskMeasure):
 
     def __lt__(self, other):
@@ -76,7 +69,7 @@ class RiskMeasure(__RiskMeasure):
             if not isinstance(other.parameters, type(self.parameters)):
                 return self.parameters.parameter_type < other.parameters.parameter_type
             else:
-                return self.parameters < other.parameters
+                return repr(self.parameters) < repr(other.parameters)
         elif other.parameters is not None:
             return True
         return False
@@ -114,4 +107,3 @@ class ParameterisedRiskMeasure(RiskMeasure):
 
     def parameter_is_empty(self):
         return self.parameters is None
-
