@@ -25,13 +25,12 @@ from dataclasses_json import global_config
 from gs_quant.api.gs.parser import GsParserApi
 from gs_quant.api.gs.risk import GsRiskApi
 from gs_quant.base import get_enum_value, InstrumentBase, Priceable, Scenario
-from gs_quant.common import AssetClass, AssetType, XRef, RiskMeasure
+from gs_quant.common import AssetClass, AssetType, XRef, RiskMeasure, MultiScenario
 from gs_quant.markets import HistoricalPricingContext, MarketDataCoordinate, PricingContext
 from gs_quant.priceable import PriceableImpl
 from gs_quant.risk import FloatWithInfo, DataFrameWithInfo, SeriesWithInfo, ResolvedInstrumentValues, \
     DEPRECATED_MEASURES
 from gs_quant.risk.results import ErrorValue, MultipleRiskMeasureFuture, PricingFuture, MultipleScenarioFuture
-from gs_quant.target.common import MultiScenario
 
 _logger = logging.getLogger(__name__)
 
@@ -40,10 +39,13 @@ class Instrument(PriceableImpl, InstrumentBase):
     PROVIDER = GsRiskApi
     __instrument_mappings = {}
 
+    def __repr__(self):
+        return f'{self.__class__.__name__}{"(" + self.name + ")" if self.name else ""}'
+
     @classmethod
     def __asset_class_and_type_to_instrument(cls):
         if not cls.__instrument_mappings:
-            import gs_quant.target.instrument as instrument_
+            import gs_quant.target.instrument as instrument_  # noqa
             instrument_classes = [c for _, c in inspect.getmembers(instrument_, inspect.isclass) if
                                   issubclass(c, Instrument) and c is not Instrument]
 
@@ -267,16 +269,16 @@ class Instrument(PriceableImpl, InstrumentBase):
     def flip(self, in_place: bool = True):
         return self.scale(-1, in_place)
 
-    def scale(self, scaling: float, in_place: bool = True):
+    def scale(self, scaling: float, in_place: bool = True, check_resolved=True):
         if scaling is None:
             return self
         if not hasattr(self, 'scale_in_place'):
             raise NotImplementedError(f'scale_in_place not implemented on {type(self).__name__}')
         if in_place:
-            self.scale_in_place(scaling)
+            self.scale_in_place(scaling, check_resolved=check_resolved)
             return
         new_inst = deepcopy(self)
-        new_inst.scale(scaling)
+        new_inst.scale(scaling, check_resolved=check_resolved)
         return new_inst
 
 
