@@ -21,15 +21,16 @@ import pandas as pd
 from typing import Dict, Optional, List, Tuple
 from pydash import get
 
-from gs_quant.api.gs.assets import Currency, GsAsset, GsAssetApi
-from gs_quant.common import DateLimit
+from gs_quant.api.gs.assets import GsAsset, GsAssetApi
+from gs_quant.api.gs.data import GsDataApi
+from gs_quant.common import AssetClass, Currency, DateLimit
 from gs_quant.data.fields import DataMeasure
 from gs_quant.entities.entity import EntityType, PositionedEntity
 from gs_quant.errors import MqValueError
 from gs_quant.instrument import Instrument
 from gs_quant.json_encoder import JSONEncoder
 from gs_quant.markets.securities import Asset, AssetType
-from gs_quant.markets.indices_utils import *
+from gs_quant.markets.indices_utils import ReturnType, STSIndexType, IndicesDatasets, PriceType
 from gs_quant.target.data import DataQuery
 from gs_quant.entities.tree_entity import AssetTreeNode, TreeHelper
 
@@ -261,7 +262,7 @@ class Index(Asset, PositionedEntity):
         if self.__is_sts_index():
             if price_type == [PriceType.INDICATIVE_CLOSE_PRICE]:
                 indicative_level = self.__query_indicative_levels_dataset(start=start, end=end)
-                indicative_level.drop(['updateTime', 'assetId'], axis=1, inplace=True)
+                indicative_level = indicative_level.drop(['updateTime', 'assetId'], axis=1)
                 indicative_level = indicative_level.astype({'date': 'datetime64[ns]'})
                 prices['date'] = indicative_level['date']
                 prices['indicativeClosePrice'] = indicative_level['indicativeClosePrice']
@@ -270,8 +271,8 @@ class Index(Asset, PositionedEntity):
             official_level = super().get_close_prices(start=start, end=end).to_frame('closePrice')
             indicative_level = self.__query_indicative_levels_dataset(start=start, end=end)
 
-            official_level.reset_index(inplace=True)
-            indicative_level.drop(['updateTime', 'assetId'], axis=1, inplace=True)
+            official_level = official_level.reset_index()
+            indicative_level = indicative_level.drop(['updateTime', 'assetId'], axis=1)
             indicative_levels = indicative_level.astype({'date': official_level.dtypes['date']})
             merged = pd.merge(official_level, indicative_levels, on='date', how='outer')
             return merged
@@ -452,7 +453,7 @@ class Index(Asset, PositionedEntity):
 
         return [position_set.get_positions() for position_set in self.get_position_sets(start, end)]
 
-    def get_latest_constituent_instruments(self) -> Tuple[Instrument]:
+    def get_latest_constituent_instruments(self) -> Tuple[Instrument, ...]:
         """
         Fetch the latest constituents of the index as instrument objects.
 
@@ -475,7 +476,7 @@ class Index(Asset, PositionedEntity):
         return GsAssetApi.get_instruments_for_positions(self.get_latest_position_set().to_target().positions)
 
     def get_constituent_instruments_for_date(self,
-                                             date: dt.date = dt.date.today()) -> Tuple[Instrument]:
+                                             date: dt.date = dt.date.today()) -> Tuple[Instrument, ...]:
         """
         Fetch the constituents of the index for a given date as instrument objects.
 
@@ -500,7 +501,7 @@ class Index(Asset, PositionedEntity):
 
     def get_constituent_instruments(self,
                                     start: dt.date = DateLimit.LOW_LIMIT.value,
-                                    end: dt.date = dt.date.today()) -> Tuple[Tuple[Instrument]]:
+                                    end: dt.date = dt.date.today()) -> Tuple[Tuple[Instrument, ...], ...]:
         """
         Fetch the constituents of the index as instrument objects for the given date range
 
