@@ -26,7 +26,7 @@ import pandas as pd
 from gs_quant.base import Priceable, RiskKey, Sentinel, InstrumentBase, is_instance_or_iterable, is_iterable, Scenario
 from gs_quant.common import RiskMeasure
 from gs_quant.config import DisplayOptions
-from gs_quant.risk import DataFrameWithInfo, ErrorValue, FloatWithInfo, SeriesWithInfo, ResultInfo, \
+from gs_quant.risk import DataFrameWithInfo, ErrorValue, UnsupportedValue, FloatWithInfo, SeriesWithInfo, ResultInfo, \
     ScalarWithInfo, aggregate_results
 from gs_quant.risk.transform import Transformer
 from more_itertools import unique_everseen
@@ -124,7 +124,13 @@ def _value_for_date(result: Union[DataFrameWithInfo, SeriesWithInfo], date: Unio
         Union[DataFrameWithInfo, ErrorValue, FloatWithInfo, SeriesWithInfo]:
     from gs_quant.markets import CloseMarket
 
-    raw_value = result.loc[date]
+    if result.empty:
+        return result
+
+    # if the result is a dataframe try to preserve the type
+    # if a dataframe has only 1 row selected it otherwise gets turned into a series
+    raw_value = result.loc[[date]] if isinstance(result, DataFrameWithInfo) and isinstance(date, dt.date) \
+        else result.loc[date]
     key = result.risk_key
 
     risk_key = RiskKey(
@@ -146,7 +152,7 @@ def _value_for_date(result: Union[DataFrameWithInfo, SeriesWithInfo], date: Unio
 
 
 def _get_value_with_info(value, risk_key, unit, error):
-    if isinstance(value, ErrorValue):
+    if isinstance(value, (ErrorValue, UnsupportedValue)):
         return value
     elif isinstance(value, pd.DataFrame):
         return DataFrameWithInfo(value, risk_key=risk_key, unit=unit, error=error)
@@ -336,7 +342,11 @@ class MultipleRiskMeasureResult(dict):
     def __op(self, operator, operand):
         values = {}
         for key, value in self.items():
-            if isinstance(value, pd.DataFrame) or isinstance(value, pd.Series):
+            if isinstance(value, SeriesWithInfo) or isinstance(value, DataFrameWithInfo):
+                new_value = value.copy_with_resultinfo()
+                if not value.empty:
+                    new_value.value = operator(value.value, operand)
+            elif isinstance(value, pd.DataFrame) or isinstance(value, pd.Series):
                 new_value = value.copy()
                 new_value.value = operator(value.value, operand)
             else:
@@ -367,7 +377,7 @@ class MultipleRiskMeasureResult(dict):
                 return tuple(value.scenarios)
         return tuple()
 
-    def to_frame(self, values='default', index='default', columns='default', aggfunc=sum,
+    def to_frame(self, values='default', index='default', columns='default', aggfunc="sum",
                  display_options: DisplayOptions = None):
         df = pd.DataFrame.from_records(self._to_records({}, display_options=display_options))
         if values is None and index is None and columns is None:
@@ -379,7 +389,7 @@ class MultipleRiskMeasureResult(dict):
                                                         has_dates='dates' in df.columns,
                                                         multi_scen='scenario' in df.columns)
         else:
-            values = 'value' if values == 'default' or values is ['value'] else values
+            values = 'value' if values == 'default' or values == ['value'] else values
             index = None if index == 'default' else index
             columns = None if columns == 'default' else columns
         return pivot_to_frame(df, values, index, columns, aggfunc)
@@ -440,7 +450,7 @@ class MultipleScenarioResult(dict):
                 raise ValueError('Can only index by date on historical results')
         return super().__getitem__(item)
 
-    def to_frame(self, values='default', index='default', columns='default', aggfunc=sum,
+    def to_frame(self, values='default', index='default', columns='default', aggfunc="sum",
                  display_options: DisplayOptions = None):
         df = pd.DataFrame.from_records(self._to_records({}, display_options=display_options))
         if values is None and index is None and columns is None:
@@ -451,7 +461,7 @@ class MultipleScenarioResult(dict):
             values, columns, index = get_default_pivots('MultipleScenarioResult', has_dates='dates' in df.columns,
                                                         multi_measures=False, multi_scen=True)
         else:
-            values = 'value' if values == 'default' or values is ['value'] else values
+            values = 'value' if values == 'default' or values == ['value'] else values
             index = None if index == 'default' else index
             columns = None if columns == 'default' else columns
         return pivot_to_frame(df, values, index, columns, aggfunc)
@@ -761,19 +771,24 @@ class PortfolioRiskResult(CompositeResultFuture):
                                              ((r, self[r].transform(risk_transformation)) for r in
                                               self.__risk_measures))
         elif len(self.__risk_measures) == 1:
-            transformed_future = PricingFuture()
-            transformed_future.set_result(risk_transformation.apply(self.__results()))
-            transformed_future.done()
-            return PortfolioRiskResult(self.portfolio, self.risk_measures, (transformed_future,))
+            flattened_results = risk_transformation.apply(self.__results())
+            futures = []
+            for result in flattened_results:
+                transformed_future = PricingFuture()
+                transformed_future.set_result(result)
+                transformed_future.done()
+                futures.append(transformed_future)
+            return PortfolioRiskResult(self.portfolio, self.risk_measures, futures)
         else:
             return self
 
-    def aggregate(self, allow_mismatch_risk_keys=False) -> Union[float, pd.DataFrame, pd.Series,
-                                                                 MultipleRiskMeasureResult]:
+    def aggregate(self, allow_mismatch_risk_keys=False,
+                  allow_heterogeneous_types=False) -> Union[float, pd.DataFrame, pd.Series, MultipleRiskMeasureResult]:
         if len(self.__risk_measures) > 1:
             return MultipleRiskMeasureResult(self.portfolio, ((r, self[r].aggregate()) for r in self.__risk_measures))
         else:
-            return aggregate_results(self.__results(), allow_mismatch_risk_keys=allow_mismatch_risk_keys)
+            return aggregate_results(self.__results(), allow_mismatch_risk_keys=allow_mismatch_risk_keys,
+                                     allow_heterogeneous_types=allow_heterogeneous_types)
 
     def _to_records(self, display_options: DisplayOptions = None):
         def get_records(rec):
@@ -794,7 +809,7 @@ class PortfolioRiskResult(CompositeResultFuture):
                 records.extend(future_records[i]._to_records({**portfolio_records[i]}, display_options))
         return records
 
-    def to_frame(self, values='default', index='default', columns='default', aggfunc=sum,
+    def to_frame(self, values='default', index='default', columns='default', aggfunc="sum",
                  display_options: DisplayOptions = None):
         final_records = self._to_records(display_options=display_options)
         if len(final_records) > 0:
@@ -832,7 +847,7 @@ class PortfolioRiskResult(CompositeResultFuture):
                                                             multi_measures=multi_rm, simple_port=port_depth_one,
                                                             multi_scen=multi_scen, ori_cols=df_cols)
         else:  # user defined pivoting
-            values = 'value' if values == 'default' or values is ['value'] else values
+            values = 'value' if values == 'default' or values == ['value'] else values
 
         return pivot_to_frame(ori_df, values, index, columns, aggfunc)
 
