@@ -35,8 +35,12 @@ class BacktestRiskMeasureType(EnumBase, Enum):
     Implied_Volatility = 'Implied Volatility'
     Fair_Variance = 'Fair Variance'
     Strike_Level = 'Strike Level'
+    Spot = 'Spot'
     Price_ATMS = 'Price ATMS'
-    Price_ATMF_Volatility = 'Price ATMF Volatility'    
+    Price_ATMF_Volatility = 'Price ATMF Volatility'
+    Barrier_Level = 'Barrier Level'
+    Strike_Level_Adj_Implied = 'Strike Level Adj Implied'
+    Strike_Level_Adj_Realised = 'Strike Level Adj Realised'    
 
 
 class BacktestTradingQuantityType(EnumBase, Enum):    
@@ -87,6 +91,7 @@ class FlowVolBacktestMeasure(EnumBase, Enum):
     PNL_theta = 'PNL_theta'
     Total = 'Total'
     transaction_costs = 'transaction_costs'
+    PNL_transactions = 'PNL_transactions'
     PNL_unexplained = 'PNL_unexplained'
     PNL_vega = 'PNL_vega'
     PNL = 'PNL'
@@ -555,6 +560,7 @@ class BacktestRiskRequest(Base):
     measures: Tuple[BacktestRiskMeasureType, ...] = field(default=None, metadata=field_metadata)
     start_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
     end_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
+    dates: Optional[Tuple[datetime.date, ...]] = field(default=None, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
@@ -610,10 +616,11 @@ class Backtest(Base):
     mq_symbol: Optional[str] = field(default=None, metadata=field_metadata)
     owner_id: Optional[str] = field(default=None, metadata=field_metadata)
     report_ids: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
-    parameters: Optional[DictBase] = field(default=None, metadata=field_metadata)
+    parameters: Optional[Union[BasketBacktestParameters, EnhancedBetaBacktestParameters, ISelectBacktestParameters, VolatilityBacktestParameters, VolatilityFlowBacktestParameters]] = field(default=None, metadata=field_metadata)
     start_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
     end_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
     version: Optional[float] = field(default=None, metadata=field_metadata)
+    cash_accrual: Optional[bool] = field(default=True, metadata=field_metadata)
 
 
 @handle_camel_case_args
