@@ -37,6 +37,31 @@ class ApprovalStatus(EnumBase, Enum):
     Error = 'Error'    
 
 
+class ISelectActionType(EnumBase, Enum):    
+    
+    """Workflow actions that exist in ISelect"""
+
+    GetStatus = 'GetStatus'
+    ValidateRebalance = 'ValidateRebalance'
+    ToggleConstraints = 'ToggleConstraints'
+    GenerateSubmissionConfirm = 'GenerateSubmissionConfirm'
+    GenerateSubmissionConfirmWithWaiver = 'GenerateSubmissionConfirmWithWaiver'
+    SubmitRebalance = 'SubmitRebalance'
+    CancelRebalance = 'CancelRebalance'
+    SubmitDraft = 'SubmitDraft'
+    CancelDraft = 'CancelDraft'
+    AgeDraft = 'AgeDraft'
+    RerunChecks = 'RerunChecks'
+    TraderApproves = 'TraderApproves'
+    TraderRejects = 'TraderRejects'
+    ClientVerifierApproves = 'ClientVerifierApproves'
+    ClientVerifierRejects = 'ClientVerifierRejects'
+    ClearException = 'ClearException'
+    ClientAcknowledges = 'ClientAcknowledges'
+    AllowWaiver = 'AllowWaiver'
+    DisableWaiver = 'DisableWaiver'    
+
+
 class IndicesCurrency(EnumBase, Enum):    
     
     """Currencies supported for Indices"""
@@ -46,6 +71,7 @@ class IndicesCurrency(EnumBase, Enum):
     GBP = 'GBP'
     CAD = 'CAD'
     AUD = 'AUD'
+    BRL = 'BRL'
     CHF = 'CHF'
     CNH = 'CNH'
     CNY = 'CNY'
@@ -71,8 +97,8 @@ class IndicesCurrency(EnumBase, Enum):
     THB = 'THB'
     TRY = 'TRY'
     TWD = 'TWD'
-    ZAR = 'ZAR'
-    BRL = 'BRL'    
+    VND = 'VND'
+    ZAR = 'ZAR'    
 
 
 @dataclass
@@ -153,6 +179,18 @@ class ISelectActionRequest(IndicesRebalanceActionTypes):
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class ISelectError(Base):
+    status_code: Optional[str] = field(default=None, metadata=config(field_name='Status_Code', exclude=exclude_none))
+    validation_messages: Optional[Tuple[str, ...]] = field(default=None, metadata=config(field_name='Validation_Messages', exclude=exclude_none))
+    date_validation_errors: Optional[Tuple[str, ...]] = field(default=None, metadata=config(field_name='Date_Validation_Errors', exclude=exclude_none))
+    index_parameters_errors: Optional[Tuple[str, ...]] = field(default=None, metadata=config(field_name='Index_Parameters_Errors', exclude=exclude_none))
+    underlyer_validation_errors: Optional[Tuple[str, ...]] = field(default=None, metadata=config(field_name='Underlyer_Validation_Errors', exclude=exclude_none))
+    name: Optional[str] = field(default=None, metadata=name_metadata)
+
+
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
@@ -197,28 +235,6 @@ class Link(Base):
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
-@handle_camel_case_args
-@dataclass_json(letter_case=LetterCase.CAMEL)
-@dataclass(unsafe_hash=True, repr=False)
-class PositionPriceInput(Base):
-    asset_id: str = field(default=None, metadata=field_metadata)
-    quantity: Optional[float] = field(default=None, metadata=field_metadata)
-    weight: Optional[float] = field(default=None, metadata=field_metadata)
-    notional: Optional[float] = field(default=None, metadata=field_metadata)
-    name: Optional[str] = field(default=None, metadata=name_metadata)
-
-
-@handle_camel_case_args
-@dataclass_json(letter_case=LetterCase.CAMEL)
-@dataclass(unsafe_hash=True, repr=False)
-class PublishParameters(Base):
-    publish_to_bloomberg: bool = field(default=False, metadata=field_metadata)
-    include_price_history: bool = field(default=False, metadata=field_metadata)
-    publish_to_reuters: Optional[bool] = field(default=False, metadata=field_metadata)
-    publish_to_factset: Optional[bool] = field(default=False, metadata=field_metadata)
-    name: Optional[str] = field(default=None, metadata=name_metadata)
-
-
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
@@ -232,16 +248,6 @@ class CreditCustomBasketPricingParameters(Base):
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
-@handle_camel_case_args
-@dataclass_json(letter_case=LetterCase.CAMEL)
-@dataclass(unsafe_hash=True, repr=False)
-class CryptoBasketRebalanceInputs(IndicesRebalanceInputTypes):
-    asset_class: AssetClass = field(default='Digital Asset', metadata=field_metadata)
-    position_set: Tuple[PositionPriceInput, ...] = field(default=None, metadata=field_metadata)
-    parameters: CryptoBasketParameters = field(default=None, metadata=field_metadata)
-    name: Optional[str] = field(default=None, metadata=name_metadata)
-
-
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
@@ -305,42 +311,6 @@ class IndicesPositionSet(Base):
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
-@handle_camel_case_args
-@dataclass_json(letter_case=LetterCase.CAMEL)
-@dataclass(unsafe_hash=True, repr=False)
-class CreditCustomBasketCreateInputs(IndicesConstructRequestTypes):
-    ticker: str = field(default=None, metadata=field_metadata)
-    name: str = field(default=None, metadata=field_metadata)
-    pricing_parameters: CreditCustomBasketPricingParameters = field(default=None, metadata=field_metadata)
-    position_set: Tuple[PositionPriceInput, ...] = field(default=None, metadata=field_metadata)
-    return_type: IndexCalculationType = field(default='Price Return', metadata=field_metadata)
-    styles: Tuple[str, ...] = field(default=None, metadata=field_metadata)
-    asset_class: Optional[AssetClass] = field(default='Credit', metadata=field_metadata)
-    description: Optional[str] = field(default=None, metadata=field_metadata)
-    related_content: Optional[GIRDomain] = field(default=None, metadata=field_metadata)
-    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
-    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
-    index_notes: Optional[str] = field(default=None, metadata=field_metadata)
-    flagship: Optional[bool] = field(default=False, metadata=field_metadata)
-    on_behalf_of: Optional[str] = field(default=None, metadata=field_metadata)
-    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
-    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
-
-
-@handle_camel_case_args
-@dataclass_json(letter_case=LetterCase.CAMEL)
-@dataclass(unsafe_hash=True, repr=False)
-class CreditCustomBasketRebalanceInputs(IndicesRebalanceInputTypes):
-    asset_class: AssetClass = field(default='Credit', metadata=field_metadata)
-    position_set: Tuple[PositionPriceInput, ...] = field(default=None, metadata=field_metadata)
-    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
-    pricing_parameters: Optional[CreditCustomBasketPricingParameters] = field(default=None, metadata=field_metadata)
-    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
-    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
-    save_as_draft: Optional[bool] = field(default=False, metadata=field_metadata)
-    name: Optional[str] = field(default=None, metadata=name_metadata)
-
-
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
@@ -356,68 +326,18 @@ class CryptoBasketCreateInputs(IndicesConstructRequestTypes):
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
-class CustomBasketsBackcastInputs(Base):
-    position_set: Tuple[IndicesPositionSet, ...] = field(default=None, metadata=field_metadata)
+class CryptoBasketRebalanceInputs(IndicesRebalanceInputTypes):
+    asset_class: AssetClass = field(default='Digital Asset', metadata=field_metadata)
+    position_set: Tuple[PositionPriceInput, ...] = field(default=None, metadata=field_metadata)
+    parameters: CryptoBasketParameters = field(default=None, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
-class CustomBasketsCreateInputs(IndicesConstructRequestTypes):
-    ticker: str = field(default=None, metadata=field_metadata)
-    name: str = field(default=None, metadata=field_metadata)
-    pricing_parameters: CustomBasketsPricingParameters = field(default=None, metadata=field_metadata)
-    position_set: Tuple[PositionPriceInput, ...] = field(default=None, metadata=field_metadata)
-    return_type: str = field(default=None, metadata=field_metadata)
-    description: Optional[str] = field(default=None, metadata=field_metadata)
-    styles: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
-    related_content: Optional[GIRDomain] = field(default=None, metadata=field_metadata)
-    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
-    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
-    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
-    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
-    index_notes: Optional[str] = field(default=None, metadata=field_metadata)
-    flagship: Optional[bool] = field(default=None, metadata=field_metadata)
-    on_behalf_of: Optional[str] = field(default=None, metadata=field_metadata)
-    allow_limited_access_assets: Optional[bool] = field(default=False, metadata=field_metadata)
-    allow_ca_restricted_assets: Optional[bool] = field(default=False, metadata=config(field_name='allowCARestrictedAssets', exclude=exclude_none))
-    vendor: Optional[str] = field(default=None, metadata=field_metadata)
-    default_backcast: Optional[bool] = field(default=True, metadata=field_metadata)
-
-
-@handle_camel_case_args
-@dataclass_json(letter_case=LetterCase.CAMEL)
-@dataclass(unsafe_hash=True, repr=False)
-class CustomBasketsEditInputs(Base):
-    name: Optional[str] = field(default=None, metadata=field_metadata)
-    description: Optional[str] = field(default=None, metadata=field_metadata)
-    styles: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
-    related_content: Optional[GIRDomain] = field(default=None, metadata=field_metadata)
-    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
-    index_notes: Optional[str] = field(default=None, metadata=field_metadata)
-    index_not_trading_reasons: Optional[IndexNotTradingReasons] = field(default=None, metadata=field_metadata)
-    flagship: Optional[bool] = field(default=None, metadata=field_metadata)
-    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
-    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
-    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
-
-
-@handle_camel_case_args
-@dataclass_json(letter_case=LetterCase.CAMEL)
-@dataclass(unsafe_hash=True, repr=False)
-class CustomBasketsRebalanceInputs(Base):
-    position_set: Optional[Tuple[PositionPriceInput, ...]] = field(default=None, metadata=field_metadata)
-    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
-    pricing_parameters: Optional[CustomBasketsPricingParameters] = field(default=None, metadata=field_metadata)
-    allow_limited_access_assets: Optional[bool] = field(default=False, metadata=field_metadata)
-    allow_ca_restricted_assets: Optional[bool] = field(default=False, metadata=config(field_name='allowCARestrictedAssets', exclude=exclude_none))
-    allow_system_approval: Optional[bool] = field(default=False, metadata=field_metadata)
-    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
-    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
-    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
-    save_as_draft: Optional[bool] = field(default=False, metadata=field_metadata)
-    allow_in_position_rebalance: Optional[bool] = field(default=False, metadata=field_metadata)
+class CustomBasketsBackcastInputs(Base):
+    position_set: Tuple[IndicesPositionSet, ...] = field(default=None, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
@@ -449,9 +369,11 @@ class DynamicConstructionResponse(IndicesConstructResponseTypes):
 class ISelectRebalance(Base):
     new_weights: Optional[Tuple[ISelectNewWeight, ...]] = field(default=None, metadata=field_metadata)
     rebalance_date: Optional[str] = field(default=None, metadata=field_metadata)
+    observation_date: Optional[str] = field(default=None, metadata=field_metadata)
     new_parameters: Optional[Tuple[ISelectNewParameter, ...]] = field(default=None, metadata=field_metadata)
     index_parameters: Optional[Tuple[ISelectIndexParameters, ...]] = field(default=None, metadata=field_metadata)
     waiver_requested: Optional[bool] = field(default=None, metadata=field_metadata)
+    unwind_missing_constituents: Optional[bool] = field(default=True, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
@@ -460,16 +382,20 @@ class ISelectRebalance(Base):
 @dataclass(unsafe_hash=True, repr=False)
 class ISelectRequest(IndicesRebalanceInputTypes):
     rebalance_date: str = field(default=None, metadata=field_metadata)
-    request_counter: int = field(default=None, metadata=field_metadata)
     use_new_rebalance_interface: bool = field(default=None, metadata=field_metadata)
     new_parameters: Optional[Tuple[ISelectNewParameter, ...]] = field(default=None, metadata=field_metadata)
     index_parameters: Optional[Tuple[ISelectIndexParameter, ...]] = field(default=None, metadata=field_metadata)
     new_weights: Optional[Tuple[ISelectNewWeight, ...]] = field(default=None, metadata=field_metadata)
     new_units: Optional[Tuple[ISelectNewUnit, ...]] = field(default=None, metadata=field_metadata)
+    observation_date: Optional[str] = field(default=None, metadata=field_metadata)
     entry_type: Optional[str] = field(default=None, metadata=field_metadata)
+    request_counter: Optional[int] = field(default=0, metadata=field_metadata)
     waiver_requested: Optional[bool] = field(default=None, metadata=field_metadata)
     presubmit: Optional[bool] = field(default=None, metadata=field_metadata)
     requester_id: Optional[str] = field(default=None, metadata=field_metadata)
+    custom_basket_import: Optional[str] = field(default=None, metadata=field_metadata)
+    action: Optional[str] = field(default=None, metadata=field_metadata)
+    unwind_missing_constituents: Optional[bool] = field(default=True, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
@@ -477,12 +403,12 @@ class ISelectRequest(IndicesRebalanceInputTypes):
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
 class ISelectResponse(Base):
-    action: Optional[object] = field(default=None, metadata=config(field_name='Action', exclude=exclude_none))
+    action: Optional[ISelectActionType] = field(default=None, metadata=config(field_name='Action', exclude=exclude_none))
     action_comment: Optional[str] = field(default=None, metadata=config(field_name='ActionComment', exclude=exclude_none))
     asset_name: Optional[str] = field(default=None, metadata=field_metadata)
     asset_short_name: Optional[str] = field(default=None, metadata=field_metadata)
     available_action_confirms: Optional[Tuple[Tuple[str, ...], ...]] = field(default=None, metadata=field_metadata)
-    available_actions: Optional[tuple] = field(default=None, metadata=field_metadata)
+    available_actions: Optional[Tuple[Union[DictBase, ISelectActionType], ...]] = field(default=None, metadata=field_metadata)
     available_rebalance_dates: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     constituent_validations: Optional[tuple] = field(default=None, metadata=field_metadata)
     date_validation_status: Optional[str] = field(default=None, metadata=field_metadata)
@@ -521,6 +447,78 @@ class IndicesDynamicConstructInputs(IndicesConstructRequestTypes):
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class PublishParameters(Base):
+    publish_to_bloomberg: bool = field(default=False, metadata=field_metadata)
+    include_price_history: bool = field(default=False, metadata=field_metadata)
+    publish_to_reuters: Optional[bool] = field(default=False, metadata=field_metadata)
+    publish_to_factset: Optional[bool] = field(default=False, metadata=field_metadata)
+    historical_methodology: Optional[EqBasketHistoryMethodology] = field(default=None, metadata=field_metadata)
+    backtest_parameters: Optional[EqBasketBacktestParameters] = field(default=None, metadata=field_metadata)
+    bloomberg_publish_parameters: Optional[BloombergPublishParameters] = field(default=None, metadata=field_metadata)
+    name: Optional[str] = field(default=None, metadata=name_metadata)
+
+
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class CreditCustomBasketCreateInputs(IndicesConstructRequestTypes):
+    ticker: str = field(default=None, metadata=field_metadata)
+    name: str = field(default=None, metadata=field_metadata)
+    pricing_parameters: CreditCustomBasketPricingParameters = field(default=None, metadata=field_metadata)
+    position_set: Tuple[PositionPriceInput, ...] = field(default=None, metadata=field_metadata)
+    return_type: IndexCalculationType = field(default='Price Return', metadata=field_metadata)
+    styles: Tuple[str, ...] = field(default=None, metadata=field_metadata)
+    asset_class: Optional[AssetClass] = field(default='Credit', metadata=field_metadata)
+    description: Optional[str] = field(default=None, metadata=field_metadata)
+    related_content: Optional[GIRDomain] = field(default=None, metadata=field_metadata)
+    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
+    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
+    index_notes: Optional[str] = field(default=None, metadata=field_metadata)
+    flagship: Optional[bool] = field(default=False, metadata=field_metadata)
+    on_behalf_of: Optional[str] = field(default=None, metadata=field_metadata)
+    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
+    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
+    credit_basket_type: Optional[CreditBasketType] = field(default=None, metadata=field_metadata)
+
+
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class CreditCustomBasketEditInputs(Base):
+    name: str = field(default=None, metadata=field_metadata)
+    description: str = field(default=None, metadata=field_metadata)
+    publish_parameters: PublishParameters = field(default=None, metadata=field_metadata)
+    pricing_parameters: CreditCustomBasketPricingParameters = field(default=None, metadata=field_metadata)
+    flagship: bool = field(default=False, metadata=field_metadata)
+    on_behalf_of: str = field(default=None, metadata=field_metadata)
+    styles: Tuple[str, ...] = field(default=None, metadata=field_metadata)
+    asset_class: Optional[AssetClass] = field(default='Credit', metadata=field_metadata)
+    related_content: Optional[GIRDomain] = field(default=None, metadata=field_metadata)
+    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
+    return_type: Optional[IndexCalculationType] = field(default='Price Return', metadata=field_metadata)
+    position_set: Optional[Tuple[PositionPriceInput, ...]] = field(default=None, metadata=field_metadata)
+    index_notes: Optional[str] = field(default=None, metadata=field_metadata)
+    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
+
+
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class CreditCustomBasketRebalanceInputs(IndicesRebalanceInputTypes):
+    asset_class: AssetClass = field(default='Credit', metadata=field_metadata)
+    position_set: Tuple[PositionPriceInput, ...] = field(default=None, metadata=field_metadata)
+    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
+    pricing_parameters: Optional[CreditCustomBasketPricingParameters] = field(default=None, metadata=field_metadata)
+    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
+    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
+    save_as_draft: Optional[bool] = field(default=False, metadata=field_metadata)
+    credit_basket_type: Optional[CreditBasketType] = field(default=None, metadata=field_metadata)
+    name: Optional[str] = field(default=None, metadata=name_metadata)
+
+
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
@@ -532,16 +530,104 @@ class IndicesBackcastInputs(Base):
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
-class IndicesEditInputs(Base):
-    parameters: CustomBasketsEditInputs = field(default=None, metadata=field_metadata)
+class CustomBasketsCreateInputs(IndicesConstructRequestTypes):
+    ticker: str = field(default=None, metadata=field_metadata)
+    name: str = field(default=None, metadata=field_metadata)
+    pricing_parameters: CustomBasketsPricingParameters = field(default=None, metadata=field_metadata)
+    position_set: Tuple[PositionPriceInput, ...] = field(default=None, metadata=field_metadata)
+    return_type: str = field(default=None, metadata=field_metadata)
+    asset_class: Optional[AssetClass] = field(default='Equity', metadata=field_metadata)
+    description: Optional[str] = field(default=None, metadata=field_metadata)
+    styles: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    related_content: Optional[GIRDomain] = field(default=None, metadata=field_metadata)
+    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
+    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
+    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
+    cash_reinvestment_treatment: Optional[CashReinvestmentTreatment] = field(default=None, metadata=field_metadata)
+    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
+    index_notes: Optional[str] = field(default=None, metadata=field_metadata)
+    flagship: Optional[bool] = field(default=None, metadata=field_metadata)
+    on_behalf_of: Optional[str] = field(default=None, metadata=field_metadata)
+    allow_limited_access_assets: Optional[bool] = field(default=False, metadata=field_metadata)
+    allow_ca_restricted_assets: Optional[bool] = field(default=False, metadata=config(field_name='allowCARestrictedAssets', exclude=exclude_none))
+    vendor: Optional[str] = field(default=None, metadata=field_metadata)
+    default_backcast: Optional[bool] = field(default=True, metadata=field_metadata)
+    action_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
+    preferred_risk_model: Optional[str] = field(default=None, metadata=field_metadata)
+    rebalance_calendar: Optional[EqBasketRebalanceCalendar] = field(default=None, metadata=field_metadata)
+    benchmark: Optional[str] = field(default=None, metadata=field_metadata)
+
+
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class CustomBasketsEditInputs(Base):
+    asset_class: Optional[AssetClass] = field(default='Equity', metadata=field_metadata)
+    name: Optional[str] = field(default=None, metadata=field_metadata)
+    description: Optional[str] = field(default=None, metadata=field_metadata)
+    styles: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    related_content: Optional[GIRDomain] = field(default=None, metadata=field_metadata)
+    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
+    index_notes: Optional[str] = field(default=None, metadata=field_metadata)
+    index_not_trading_reasons: Optional[IndexNotTradingReasons] = field(default=None, metadata=field_metadata)
+    flagship: Optional[bool] = field(default=None, metadata=field_metadata)
+    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
+    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
+    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
+    vendor: Optional[str] = field(default=None, metadata=field_metadata)
+    action_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
+    preferred_risk_model: Optional[str] = field(default=None, metadata=field_metadata)
+    rebalance_calendar: Optional[EqBasketRebalanceCalendar] = field(default=None, metadata=field_metadata)
+    benchmark: Optional[str] = field(default=None, metadata=field_metadata)
+    on_behalf_of: Optional[str] = field(default=None, metadata=field_metadata)
+    cash_reinvestment_treatment: Optional[CashReinvestmentTreatment] = field(default=None, metadata=field_metadata)
+
+
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class CustomBasketsRebalanceInputs(Base):
+    asset_class: Optional[AssetClass] = field(default='Equity', metadata=field_metadata)
+    position_set: Optional[Tuple[PositionPriceInput, ...]] = field(default=None, metadata=field_metadata)
+    publish_parameters: Optional[PublishParameters] = field(default=None, metadata=field_metadata)
+    pricing_parameters: Optional[CustomBasketsPricingParameters] = field(default=None, metadata=field_metadata)
+    allow_limited_access_assets: Optional[bool] = field(default=False, metadata=field_metadata)
+    allow_ca_restricted_assets: Optional[bool] = field(default=False, metadata=config(field_name='allowCARestrictedAssets', exclude=exclude_none))
+    allow_system_approval: Optional[bool] = field(default=False, metadata=field_metadata)
+    clone_parent_id: Optional[str] = field(default=None, metadata=field_metadata)
+    hedge_id: Optional[str] = field(default=None, metadata=field_metadata)
+    portfolio_id: Optional[str] = field(default=None, metadata=field_metadata)
+    save_as_draft: Optional[bool] = field(default=False, metadata=field_metadata)
+    allow_in_position_rebalance: Optional[bool] = field(default=False, metadata=field_metadata)
+    action_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
+    preferred_risk_model: Optional[str] = field(default=None, metadata=field_metadata)
+    rebalance_calendar: Optional[EqBasketRebalanceCalendar] = field(default=None, metadata=field_metadata)
+    benchmark: Optional[str] = field(default=None, metadata=field_metadata)
+    on_behalf_of: Optional[str] = field(default=None, metadata=field_metadata)
+    cash_reinvestment_treatment: Optional[CashReinvestmentTreatment] = field(default=None, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class IndicesEditInputTypes(Base):
+    parameters: Union[CreditCustomBasketEditInputs, CustomBasketsEditInputs] = field(default=None, metadata=field_metadata)
+    name: Optional[str] = field(default=None, metadata=name_metadata)
+
+
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class IndicesEditInputs(Base):
+    parameters: CustomBasketsEditInputs = field(default=None, metadata=field_metadata)
+
+
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
 class IndicesRebalanceInputs(IndicesRebalanceInputTypes):
-    parameters: DictBase = field(default=None, metadata=field_metadata)
+    parameters: Union[CustomBasketsRebalanceInputs, ISelectRebalance] = field(default=None, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
