@@ -69,11 +69,25 @@ class SecMasterAssetType(EnumBase, Enum):
     Financial_index_future_ = 'Financial index future.'
     Single_Stock_Future = 'Single Stock Future'
     Pool = 'Pool'
-    Certificate_of_Deposit = 'Certificate of Deposit'
+    Certificate_Of_Deposit = 'Certificate Of Deposit'
     Debt_Structured_Note = 'Debt Structured Note'
     Agency_CMO = 'Agency CMO'
     Future_Option = 'Future Option'
-    Convertible_Bond = 'Convertible Bond'    
+    Convertible_Bond = 'Convertible Bond'
+    Austrian_Crt = 'Austrian Crt'
+    BDR = 'BDR'
+    Belgium_Cert = 'Belgium Cert'
+    CDR = 'CDR'
+    EDR = 'EDR'
+    German_Cert = 'German Cert'
+    IDR = 'IDR'
+    RDC = 'RDC'
+    Swiss_Cert = 'Swiss Cert'
+    Canadian_DR = 'Canadian DR'
+    Singapore_DR = 'Singapore DR'
+    To_Be_Announced = 'To Be Announced'
+    Basket = 'Basket'
+    Physical_index_future_ = 'Physical index future.'
 
 
 class SecMasterCorporateActionStatus(EnumBase, Enum):    
@@ -91,7 +105,8 @@ class SecMasterCorporateActionStatus(EnumBase, Enum):
     Cancelled = 'Cancelled'
     Lapsed = 'Lapsed'
     Proposed = 'Proposed'
-    Withdrawn = 'Withdrawn'    
+    Withdrawn = 'Withdrawn'
+    All = 'All'    
 
 
 class SecMasterCorporateActionType(EnumBase, Enum):    
@@ -101,7 +116,13 @@ class SecMasterCorporateActionType(EnumBase, Enum):
     Cash_Dividend = 'Cash Dividend'
     Merger = 'Merger'
     Spinoff = 'Spinoff'
-    Split = 'Split'    
+    Split = 'Split'
+    Rights = 'Rights'
+    Reorganization = 'Reorganization'
+    Special_Adjustment = 'Special Adjustment'
+    Quote_Lot_Adjustment = 'Quote Lot Adjustment'
+    Currency_Adjustment = 'Currency Adjustment'
+    All = 'All'    
 
 
 @handle_camel_case_args
@@ -116,7 +137,8 @@ class SecMasterAuditFields(Base):
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
-SecMasterIdentifiers = Dict[str, str]
+class SecMasterIdentifiers(DictBase):
+    pass
 
 
 @handle_camel_case_args
@@ -137,7 +159,9 @@ class SecMasterRecord(Base):
 class SecMasterResourceCompany(Base):
     company_id: Optional[float] = field(default=None, metadata=field_metadata)
     company_name: Optional[str] = field(default=None, metadata=field_metadata)
-    name: Optional[str] = field(default=None, metadata=name_metadata)
+    identifiers: Optional[DictBase] = field(default=None, metadata=field_metadata)
+    name: Optional[str] = field(default=None, metadata=field_metadata)
+    issuer_id: Optional[str] = field(default=None, metadata=field_metadata)
 
 
 SecMasterSources = Dict[str, str]
@@ -146,7 +170,7 @@ SecMasterSources = Dict[str, str]
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
-class SecMasterTemporalCompany(SecMasterResponseMulti):
+class SecMasterTemporalCompany(Base):
     gs_company_id: str = field(default=None, metadata=field_metadata)
     id_: Optional[str] = field(default=None, metadata=config(field_name='id', exclude=exclude_none))
     start_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
@@ -207,7 +231,7 @@ class SecMasterCorporateAction(Base):
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
-class SecMasterExchange(SecMasterResponseMulti):
+class SecMasterExchange(Base):
     gs_exchange_id: str = field(default=None, metadata=field_metadata)
     id_: Optional[str] = field(default=None, metadata=config(field_name='id', exclude=exclude_none))
     name: Optional[str] = field(default=None, metadata=field_metadata)
@@ -227,13 +251,18 @@ class SecMasterGetActionsRequestPathSchema(Base):
     event_id: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     corp_action_id: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     effective_date: Optional[Tuple[datetime.date, ...]] = field(default=None, metadata=field_metadata)
+    as_of_time: Optional[Tuple[datetime.datetime, ...]] = field(default=None, metadata=field_metadata)
+    effective_date_from: Optional[Tuple[datetime.date, ...]] = field(default=None, metadata=field_metadata)
+    effective_date_to: Optional[Tuple[datetime.date, ...]] = field(default=None, metadata=field_metadata)
+    offset_key: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    limit: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
-class SecMasterGetRequestPathSchema(Base):
+class SecMasterGetCapitalStructureRequestPathSchema(Base):
     gsid: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     ticker: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     bbid: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
@@ -244,12 +273,40 @@ class SecMasterGetRequestPathSchema(Base):
     isin: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     gss: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     prime_id: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    issuer_id: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     type_: Optional[Tuple[str, ...]] = field(default=None, metadata=config(field_name='type', exclude=exclude_none))
-    exchange: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
-    fields: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
-    as_of_date: Optional[Tuple[datetime.date, ...]] = field(default=None, metadata=field_metadata)
+    as_of_time: Optional[Tuple[datetime.datetime, ...]] = field(default=None, metadata=field_metadata)
+    is_primary: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    effective_date: Optional[Tuple[datetime.date, ...]] = field(default=None, metadata=field_metadata)
+    limit: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    offset_key: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    name: Optional[str] = field(default=None, metadata=name_metadata)
+
+
+@handle_camel_case_args
+@dataclass_json(letter_case=LetterCase.CAMEL)
+@dataclass(unsafe_hash=True, repr=False)
+class SecMasterGetRequestPathSchema(Base):
+    identifier: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    gsid: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    ticker: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    bbg: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    bbid: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    ric: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    rcic: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    cusip: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    cins: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    sedol: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    isin: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    gss: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    prime_id: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    type_: Optional[Tuple[SecMasterAssetType, ...]] = field(default=None, metadata=config(field_name='type', exclude=exclude_none))
+    country_code: Optional[Tuple[CountryCode, ...]] = field(default=None, metadata=field_metadata)
+    is_primary: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    all_listings: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    effective_date: Optional[Tuple[datetime.date, ...]] = field(default=None, metadata=field_metadata)
+    as_of_time: Optional[Tuple[datetime.datetime, ...]] = field(default=None, metadata=field_metadata)
     limit: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
-    offset: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     offset_key: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
 
@@ -273,7 +330,7 @@ class SecMasterResourceProduct(Base):
 @handle_camel_case_args
 @dataclass_json(letter_case=LetterCase.CAMEL)
 @dataclass(unsafe_hash=True, repr=False)
-class SecMasterTemporalProduct(SecMasterResponseMulti):
+class SecMasterTemporalProduct(Base):
     gsid: str = field(default=None, metadata=field_metadata)
     id_: Optional[str] = field(default=None, metadata=config(field_name='id', exclude=exclude_none))
     start_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
@@ -293,19 +350,25 @@ class SecMasterTemporalProduct(SecMasterResponseMulti):
 @dataclass(unsafe_hash=True, repr=False)
 class SecMasterAsset(Base):
     id_: str = field(default=None, metadata=config(field_name='id', exclude=exclude_none))
+    effective_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
+    start_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
+    name: Optional[str] = field(default=None, metadata=field_metadata)
     asset_class: Optional[AssetClass] = field(default=None, metadata=field_metadata)
     type_: Optional[SecMasterAssetType] = field(default=None, metadata=config(field_name='type', exclude=exclude_none))
     product: Optional[SecMasterResourceProduct] = field(default=None, metadata=field_metadata)
     exchange: Optional[SecMasterResourceExchange] = field(default=None, metadata=field_metadata)
+    currency: Optional[str] = field(default=None, metadata=field_metadata)
     company: Optional[SecMasterResourceCompany] = field(default=None, metadata=field_metadata)
+    issuer: Optional[SecMasterResourceCompany] = field(default=None, metadata=field_metadata)
     classifications: Optional[AssetClassifications] = field(default=None, metadata=field_metadata)
     identifiers: Optional[SecMasterIdentifiers] = field(default=None, metadata=field_metadata)
     tags: Optional[Tuple[str, ...]] = field(default=None, metadata=field_metadata)
+    active_listing: Optional[bool] = field(default=None, metadata=field_metadata)
+    last_active_date: Optional[datetime.date] = field(default=None, metadata=field_metadata)
     entitlements: Optional[Entitlements] = field(default=None, metadata=field_metadata)
     entitlement_exclusions: Optional[EntitlementExclusions] = field(default=None, metadata=field_metadata)
     audit_fields: Optional[SecMasterAuditFields] = field(default=None, metadata=field_metadata)
     field_sources: Optional[SecMasterAssetSources] = field(default=None, metadata=field_metadata)
-    name: Optional[str] = field(default=None, metadata=name_metadata)
 
 
 @handle_camel_case_args
@@ -323,7 +386,7 @@ class SecMasterResponseActions(Base):
 @dataclass(unsafe_hash=True, repr=False)
 class SecMasterResponseMulti(Base):
     request_id: Optional[str] = field(default=None, metadata=field_metadata)
-    results: Optional[Tuple[SecMasterResponseMulti, ...]] = field(default=None, metadata=field_metadata)
+    results: Optional[Tuple[Union[SecMasterExchange, SecMasterTemporalCompany, SecMasterTemporalProduct], ...]] = field(default=None, metadata=field_metadata)
     total_results: Optional[float] = field(default=None, metadata=field_metadata)
     offset_key: Optional[str] = field(default=None, metadata=field_metadata)
     limit: Optional[int] = field(default=None, metadata=field_metadata)
@@ -340,4 +403,5 @@ class SecMasterResponseAssets(Base):
     offset_key: Optional[str] = field(default=None, metadata=field_metadata)
     limit: Optional[int] = field(default=None, metadata=field_metadata)
     offset: Optional[int] = field(default=None, metadata=field_metadata)
+    update_time: Optional[datetime.datetime] = field(default=None, metadata=field_metadata)
     name: Optional[str] = field(default=None, metadata=name_metadata)
