@@ -18,21 +18,25 @@ from unittest import mock
 
 import datetime as dt
 from gs_quant.api.gs.backtests import GsBacktestApi
+from gs_quant.backtests import TradeInMethod, Backtest
+from gs_quant.backtests.backtest_objects import ConstantTransactionModel
 from gs_quant.backtests.strategy import Strategy
-from gs_quant.backtests.triggers import PeriodicTrigger, PeriodicTriggerRequirements, DateTrigger, \
-    DateTriggerRequirements, AggregateTrigger, PortfolioTrigger, PortfolioTriggerRequirements, \
-    TriggerDirection
-from gs_quant.backtests.actions import EnterPositionQuantityScaledAction, HedgeAction, ExitPositionAction
-from gs_quant.backtests.equity_vol_engine import *
-from gs_quant.common import Currency, AssetClass
+from gs_quant.backtests.triggers import PeriodicTrigger, PeriodicTriggerRequirements, DateTriggerRequirements, \
+    AggregateTrigger, AggregateTriggerRequirements, PortfolioTriggerRequirements, TriggerDirection
+from gs_quant.backtests.actions import EnterPositionQuantityScaledAction, HedgeAction, ExitPositionAction, \
+    AddTradeAction, AddScaledTradeAction, ScalingActionType
+from gs_quant.backtests.equity_vol_engine import EquityVolEngine
+from gs_quant.common import BuySell, Currency, AssetClass
+from gs_quant.instrument import EqOption
+from gs_quant.markets.portfolio import Portfolio
+from gs_quant.risk import EqDelta
 from gs_quant.session import GsSession, Environment
-from gs_quant.target.backtests import BacktestResult as APIBacktestResult, Backtest, OptionStyle, OptionType, \
+from gs_quant.target.backtests import BacktestResult as APIBacktestResult, OptionStyle, OptionType, \
     BacktestTradingParameters, BacktestStrategyUnderlier, BacktestStrategyUnderlierHedge, \
-    VolatilityFlowBacktestParameters, BacktestTradingQuantityType, BacktestSignalSeriesItem
+    VolatilityFlowBacktestParameters, BacktestTradingQuantityType, BacktestSignalSeriesItem, FlowVolBacktestMeasure, \
+    DeltaHedgeParameters, EquityMarketModel
 import pandas as pd
 
-from gs_quant.target.common import BuySell
-
 
 def set_session():
     from gs_quant.session import OAuth2Session
@@ -80,22 +84,22 @@ def test_eq_vol_engine_result(mocker):
     end_date = dt.date(2019, 2, 20)
 
     option = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                      option_style=OptionStyle.European)
+                      option_style=OptionStyle.European, name='option')
 
     long_call = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Buy, number_of_options=1)
     short_put = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Put,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Sell)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Sell, number_of_options=1)
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedgetrigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedgetrigger])
 
     # 2. setup mock api response
@@ -126,22 +130,22 @@ def test_engine_mapping_basic(mocker):
     end_date = dt.date(2019, 2, 20)
 
     option = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                      option_style=OptionStyle.European)
+                      option_style=OptionStyle.European, number_of_options=1, name='option')
 
     long_call = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Buy, number_of_options=1)
     short_put = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Put,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Sell)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Sell, number_of_options=1)
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedgetrigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedgetrigger])
 
     # 2. setup mock api response
@@ -155,6 +159,7 @@ def test_engine_mapping_basic(mocker):
 
     # 4. assert API call
 
+    option.number_of_options = 1
     backtest_parameter_args = {
         'trading_parameters': BacktestTradingParameters(
             quantity=1,
@@ -194,23 +199,23 @@ def test_engine_mapping_trade_quantity(mocker):
     end_date = dt.date(2019, 2, 20)
 
     option = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                      option_style=OptionStyle.European)
+                      option_style=OptionStyle.European, number_of_options=1, name='option')
 
     long_call = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Buy, number_of_options=1)
     short_put = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Put,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Sell)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Sell, number_of_options=1)
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
 
     action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', trade_quantity=12345,
-                                               trade_quantity_type=BacktestTradingQuantityType.notional)
+                                               trade_quantity_type=BacktestTradingQuantityType.notional, name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedgetrigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedgetrigger])
 
     # 2. setup mock api response
@@ -224,6 +229,7 @@ def test_engine_mapping_trade_quantity(mocker):
 
     # 4. assert API call
 
+    option.number_of_options = 1
     backtest_parameter_args = {
         'trading_parameters': BacktestTradingParameters(
             quantity=12345,
@@ -263,26 +269,25 @@ def test_engine_mapping_with_signals(mocker):
     end_date = dt.date(2019, 2, 27)
 
     option = EqOption('.STOXX50E', expirationDate='3m', strikePrice='ATM', optionType=OptionType.Call,
-                      optionStyle=OptionStyle.European)
+                      optionStyle=OptionStyle.European, name='option', number_of_options=1)
 
     entry_action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', trade_quantity=12345,
-                                                     trade_quantity_type=BacktestTradingQuantityType.notional)
+                                                     trade_quantity_type=BacktestTradingQuantityType.notional,
+                                                     name='action')
 
     entry_signal_series = pd.Series(data={dt.date(2019, 2, 19): 1})
     entry_dates = entry_signal_series[entry_signal_series > 0].keys()
 
-    entry_trigger = AggregateTrigger(triggers=[
-        DateTrigger(trigger_requirements=DateTriggerRequirements(dates=entry_dates), actions=entry_action),
-        PortfolioTrigger(trigger_requirements=PortfolioTriggerRequirements('len', 0, TriggerDirection.EQUAL))
-    ])
+    entry_trigger = AggregateTrigger(AggregateTriggerRequirements(triggers=[
+        DateTriggerRequirements(dates=entry_dates), PortfolioTriggerRequirements('len', 0, TriggerDirection.EQUAL)]),
+        actions=entry_action)
 
     exit_signal_series = pd.Series(data={dt.date(2019, 2, 20): 1})
     exit_dates = exit_signal_series[exit_signal_series > 0].keys()
 
-    exit_trigger = AggregateTrigger(triggers=[
-        DateTrigger(trigger_requirements=DateTriggerRequirements(dates=exit_dates), actions=ExitPositionAction()),
-        PortfolioTrigger(trigger_requirements=PortfolioTriggerRequirements('len', 0, TriggerDirection.ABOVE))
-    ])
+    exit_trigger = AggregateTrigger(AggregateTriggerRequirements(triggers=[
+        DateTriggerRequirements(dates=exit_dates), PortfolioTriggerRequirements('len', 0, TriggerDirection.ABOVE)]),
+        actions=ExitPositionAction(name='exit_action'))
 
     strategy = Strategy(initial_portfolio=None, triggers=[entry_trigger, exit_trigger])
 
@@ -297,6 +302,7 @@ def test_engine_mapping_with_signals(mocker):
 
     # 4. assert API call
 
+    option.number_of_options = 1
     backtest_parameter_args = {
         'trading_parameters': BacktestTradingParameters(
             quantity=12345,
@@ -343,23 +349,24 @@ def test_engine_mapping_trade_quantity_nav(mocker):
     end_date = dt.date(2019, 2, 20)
 
     option = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                      option_style=OptionStyle.European)
+                      option_style=OptionStyle.European, number_of_options=1, name='option')
 
     long_call = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Buy, number_of_options=1)
     short_put = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Put,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Sell)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Sell, number_of_options=1)
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
 
     action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', trade_quantity=12345,
-                                               trade_quantity_type=BacktestTradingQuantityType.NAV)
+                                               trade_quantity_type=BacktestTradingQuantityType.NAV,
+                                               name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedgetrigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedgetrigger])
 
     # 2. setup mock api response
@@ -373,6 +380,7 @@ def test_engine_mapping_trade_quantity_nav(mocker):
 
     # 4. assert API call
 
+    option.number_of_options = 1
     backtest_parameter_args = {
         'trading_parameters': BacktestTradingParameters(
             quantity=12345,
@@ -412,22 +420,22 @@ def test_engine_mapping_listed(mocker):
     end_date = dt.date(2019, 2, 20)
 
     option = EqOption('.STOXX50E', expiration_date='3m@listed', strike_price='ATM', option_type=OptionType.Call,
-                      option_style=OptionStyle.European)
+                      option_style=OptionStyle.European, number_of_options=1, name='option')
 
     long_call = EqOption('.STOXX50E', expiration_date='3m@listed', strike_price='ATM', option_type=OptionType.Call,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Buy, number_of_options=1)
     short_put = EqOption('.STOXX50E', expiration_date='3m@listed', strike_price='ATM', option_type=OptionType.Put,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Sell)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Sell, number_of_options=1)
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedgetrigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedgetrigger])
 
     # 2. setup mock api response
@@ -442,6 +450,7 @@ def test_engine_mapping_listed(mocker):
     # 4. assert API call
 
     option.expiration_date = '3m'
+    option.number_of_options = 1
     backtest_parameter_args = {
         'trading_parameters': BacktestTradingParameters(
             quantity=1,
@@ -481,22 +490,22 @@ def test_engine_mapping_market_model(mocker):
     end_date = dt.date(2019, 2, 20)
 
     option = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                      option_style=OptionStyle.European)
+                      option_style=OptionStyle.European, number_of_options=1, name='option')
 
     long_call = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Buy, number_of_options=1)
     short_put = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Put,
-                         option_style=OptionStyle.European, buy_sell=BuySell.Sell)
+                         option_style=OptionStyle.European, buy_sell=BuySell.Sell, number_of_options=1)
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedgetrigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedgetrigger])
 
     # 2. setup mock api response
@@ -510,6 +519,7 @@ def test_engine_mapping_market_model(mocker):
 
     # 4. assert response
 
+    option.number_of_options = 1
     backtest_parameter_args = {
         'trading_parameters': BacktestTradingParameters(
             quantity=1,
@@ -549,10 +559,10 @@ def test_engine_mapping_portfolio(mocker):
     end_date = dt.date(2019, 2, 20)
 
     call = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                    option_style=OptionStyle.European)
+                    option_style=OptionStyle.European, number_of_options=1, name='option_call')
 
     put = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Put,
-                   option_style=OptionStyle.European)
+                   option_style=OptionStyle.European, number_of_options=1, name='option_put')
 
     portfolio = Portfolio(name='portfolio', priceables=[call, put])
 
@@ -563,13 +573,13 @@ def test_engine_mapping_portfolio(mocker):
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
 
-    action = EnterPositionQuantityScaledAction(priceables=portfolio, trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=portfolio, trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedgetrigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedgetrigger])
 
     # 2. setup mock api response
@@ -583,6 +593,8 @@ def test_engine_mapping_portfolio(mocker):
 
     # 4. assert response
 
+    call.number_of_options = 1
+    put.number_of_options = 1
     backtest_parameter_args = {
         'trading_parameters': BacktestTradingParameters(
             quantity=1,
@@ -627,7 +639,7 @@ def test_supports_strategy():
     end_date = dt.date(2019, 2, 20)
 
     option = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                      option_style=OptionStyle.European)
+                      option_style=OptionStyle.European, name='option')
 
     long_call = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
                          option_style=OptionStyle.European, buy_sell=BuySell.Buy)
@@ -636,13 +648,13 @@ def test_supports_strategy():
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedge_trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedge_trigger])
 
     assert EquityVolEngine.supports_strategy(strategy)
@@ -658,7 +670,8 @@ def test_supports_strategy():
 
     # 3. Invalid - no trade quantity
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', trade_quantity=None)
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', trade_quantity=None,
+                                               name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
@@ -667,7 +680,8 @@ def test_supports_strategy():
 
     # 4. Invalid - no trade quantity type
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', trade_quantity_type=None)
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', trade_quantity_type=None,
+                                               name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
@@ -676,7 +690,7 @@ def test_supports_strategy():
 
     # 5. Invalid - mismatch trade duration and trigger period
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='2m')
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='2m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
@@ -685,100 +699,229 @@ def test_supports_strategy():
 
     # 6. Invalid - mismatch hedge trade duration and trigger period
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedge_trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='B'),
-        actions=HedgeAction(EqDelta, priceables=option, trade_duration='M'))
+        actions=HedgeAction(EqDelta, priceables=option, trade_duration='M', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedge_trigger])
     assert not EquityVolEngine.supports_strategy(strategy)
 
-    # 6. Invalid - non-daily hedge trade
+    # 7. Invalid - non-daily hedge trade
 
-    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     hedge_trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='M'),
-        actions=HedgeAction(EqDelta, priceables=option, trade_duration='M'))
+        actions=HedgeAction(EqDelta, priceables=option, trade_duration='M', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedge_trigger])
     assert not EquityVolEngine.supports_strategy(strategy)
 
-    # 7. Invalid - expiration date modifiers must be the same
+    # 8. Invalid - expiration date modifiers must be the same
 
     option_listed = EqOption('.STOXX50E', expirationDate='3m@listed', strikePrice='ATM', optionType=OptionType.Call,
-                             optionStyle=OptionStyle.European)
+                             optionStyle=OptionStyle.European, name='option')
 
-    action = EnterPositionQuantityScaledAction(priceables=[option, option_listed], trade_duration='1m')
+    action = EnterPositionQuantityScaledAction(priceables=[option, option_listed], trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     strategy = Strategy(initial_portfolio=None, triggers=[trigger])
     assert not EquityVolEngine.supports_strategy(strategy)
 
-    option_listed = EqOption('.STOXX50E', expirationDate='3m@listed', strikePrice='ATM', optionType=OptionType.Call,
-                             optionStyle=OptionStyle.European)
+    # 9. Invalid - expiration date modifier not in [otc, listed]
 
-    # 8. Invalid - expiration date modifier not in [otc, listed]
-
-    option_invalid = EqOption('.STOXX50E', expirationDate='3m@invalid', strikePrice='ATM', optionType=OptionType.Call)
-    action = EnterPositionQuantityScaledAction(priceables=[option_invalid], trade_duration='1m')
+    option_invalid = EqOption('.STOXX50E', expirationDate='3m@invalid', strikePrice='ATM', optionType=OptionType.Call,
+                              name='option')
+    action = EnterPositionQuantityScaledAction(priceables=[option_invalid], trade_duration='1m', name='action')
     trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
         actions=action)
     strategy = Strategy(initial_portfolio=None, triggers=[trigger])
     assert not EquityVolEngine.supports_strategy(strategy)
 
-    # 9. Invalid - hedging without synthetic forward (not a portfolio)
+    # 10. Invalid - hedging without synthetic forward (not a portfolio)
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call])
 
     hedge_trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedge_trigger])
 
     assert not EquityVolEngine.supports_strategy(strategy)
 
-    # 10. Invalid - hedging without synthetic forward (two calls)
+    # 11. Invalid - hedging without synthetic forward (two calls)
 
     long_call_2 = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
-                           option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+                           option_style=OptionStyle.European, buy_sell=BuySell.Buy, name='option')
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, long_call_2])
 
     hedge_trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedge_trigger])
 
     assert not EquityVolEngine.supports_strategy(strategy)
 
-    # 10. Invalid - hedging without synthetic forward (more than two options)
+    # 12. Invalid - hedging without synthetic forward (more than two options)
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put, long_call_2])
 
     hedge_trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedge_trigger])
 
     assert not EquityVolEngine.supports_strategy(strategy)
 
-    # 10. Invalid - hedging without synthetic forward (properties mismatch)
+    # 13. Invalid - hedging without synthetic forward (properties mismatch)
 
     long_call_2m = EqOption('.STOXX50E', expiration_date='2m', strike_price='ATM', option_type=OptionType.Call,
-                            option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+                            option_style=OptionStyle.European, buy_sell=BuySell.Buy, name='option_1')
     short_put_4m = EqOption('.STOXX50E', expiration_date='4m', strike_price='ATM', option_type=OptionType.Put,
-                            option_style=OptionStyle.European, buy_sell=BuySell.Sell)
+                            option_style=OptionStyle.European, buy_sell=BuySell.Sell, name='option_2')
 
     hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call_2m, short_put_4m])
 
     hedge_trigger = PeriodicTrigger(
         trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
-        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b'))
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
     strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedge_trigger])
 
     assert not EquityVolEngine.supports_strategy(strategy)
+
+    # 14. Valid - AddTradeAction
+
+    add_trade_action = AddTradeAction(priceables=option, trade_duration='1m', name='action')
+    trigger = PeriodicTrigger(
+        trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
+        actions=add_trade_action)
+    strategy = Strategy(initial_portfolio=None, triggers=[trigger])
+    assert EquityVolEngine.supports_strategy(strategy)
+
+    # 15. Valid - AddScaledTradeAction
+
+    add_scaled_trade_action = AddScaledTradeAction(priceables=option, trade_duration='1m',
+                                                   scaling_type=ScalingActionType.size, scaling_level=2, name='action')
+    trigger = PeriodicTrigger(
+        trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
+        actions=add_scaled_trade_action)
+    strategy = Strategy(initial_portfolio=None, triggers=[trigger])
+    assert EquityVolEngine.supports_strategy(strategy)
+
+    # 16. Invalid - transaction_costs not supported
+
+    add_trade_action_tc = AddTradeAction(priceables=option, trade_duration='1m',
+                                         transaction_cost=ConstantTransactionModel(100), name='action')
+    trigger = PeriodicTrigger(
+        trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
+        actions=add_trade_action_tc)
+    strategy = Strategy(initial_portfolio=None, triggers=[trigger])
+    assert not EquityVolEngine.supports_strategy(strategy)
+
+    # 17. Valid - transaction_costs supported if 0
+
+    add_trade_action_tc0 = AddTradeAction(priceables=option, trade_duration='1m',
+                                          transaction_cost=ConstantTransactionModel(0), name='action')
+    trigger = PeriodicTrigger(
+        trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
+        actions=add_trade_action_tc0)
+    strategy = Strategy(initial_portfolio=None, triggers=[trigger])
+    assert EquityVolEngine.supports_strategy(strategy)
+
+    # 18. Invalid - instrument non unit contract size
+
+    option_with_mult = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
+                                option_style=OptionStyle.European, multiplier=100, name='option')
+
+    action = AddTradeAction(priceables=option_with_mult, trade_duration='1m', name='action')
+    trigger = PeriodicTrigger(
+        trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
+        actions=action)
+    strategy = Strategy(initial_portfolio=None, triggers=[trigger])
+    assert not EquityVolEngine.supports_strategy(strategy)
+
+    # 19. Invalid - instrument non unit contract count
+
+    option_with_contracts = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
+                                     option_style=OptionStyle.European, number_of_options=100, name='option')
+
+    action = AddTradeAction(priceables=option_with_contracts, trade_duration='1m', name='action')
+    trigger = PeriodicTrigger(
+        trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
+        actions=action)
+    strategy = Strategy(initial_portfolio=None, triggers=[trigger])
+    assert not EquityVolEngine.supports_strategy(strategy)
+
+
+@mock.patch.object(GsBacktestApi, 'run_backtest')
+def test_engine_mapping_basic_leg_size(mocker):
+    # 1. setup strategy
+
+    start_date = dt.date(2019, 2, 18)
+    end_date = dt.date(2019, 2, 20)
+
+    option = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
+                      option_style=OptionStyle.European, name='option', number_of_options=0)
+
+    long_call = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Call,
+                         option_style=OptionStyle.European, buy_sell=BuySell.Buy)
+    short_put = EqOption('.STOXX50E', expiration_date='3m', strike_price='ATM', option_type=OptionType.Put,
+                         option_style=OptionStyle.European, buy_sell=BuySell.Sell)
+
+    hedge_portfolio = Portfolio(name='SynFwd', priceables=[long_call, short_put])
+
+    action = EnterPositionQuantityScaledAction(priceables=option, trade_duration='1m', name='action')
+    trigger = PeriodicTrigger(
+        trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1m'),
+        actions=action)
+    hedgetrigger = PeriodicTrigger(
+        trigger_requirements=PeriodicTriggerRequirements(start_date=start_date, end_date=end_date, frequency='1b'),
+        actions=HedgeAction(EqDelta, priceables=hedge_portfolio, trade_duration='1b', name='hedge_action'))
+    strategy = Strategy(initial_portfolio=None, triggers=[trigger, hedgetrigger])
+
+    # 2. setup mock api response
+
+    mock_api_response(mocker, api_mock_data())
+
+    # 3. when run backtest
+
+    set_session()
+    EquityVolEngine.run_backtest(strategy, start_date, end_date)
+
+    # 4. assert API call
+
+    backtest_parameter_args = {
+        'trading_parameters': BacktestTradingParameters(
+            quantity=1,
+            quantity_type=BacktestTradingQuantityType.quantity.value,
+            trade_in_method=TradeInMethod.FixedRoll.value,
+            roll_frequency='1m'),
+        'underliers': [BacktestStrategyUnderlier(
+            instrument=option,
+            notional_percentage=100,
+            hedge=BacktestStrategyUnderlierHedge(risk_details=DeltaHedgeParameters(frequency='Daily')),
+            market_model='SFK',
+            expiry_date_mode='otc')
+        ],
+        'index_initial_value': 0.0,
+        "measures": [FlowVolBacktestMeasure.ALL_MEASURES]
+    }
+    backtest_parameters = VolatilityFlowBacktestParameters.from_dict(backtest_parameter_args)
+
+    backtest = Backtest(name="Flow Vol Backtest",
+                        mq_symbol="Flow Vol Backtest",
+                        parameters=backtest_parameters,
+                        start_date=start_date,
+                        end_date=end_date,
+                        type='Volatility Flow',
+                        asset_class=AssetClass.Equity,
+                        currency=Currency.USD,
+                        cost_netting=False)
+
+    mocker.assert_called_with(backtest, None)
