@@ -13,13 +13,12 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
-import datetime
+import datetime as dt
 from enum import IntEnum
 from unittest.mock import Mock
 
 import pandas as pd
 import pytest
-from pandas.testing import assert_frame_equal
 from testfixtures import Replace, Replacer
 
 import gs_quant.timeseries as ts
@@ -204,8 +203,8 @@ def test_apply_ramp_raises_on_edge_cases():
 
 
 def test_get_df_with_retries():
-    start = datetime.date(2020, 12, 1)
-    end = datetime.date(2020, 12, 2)
+    start = dt.date(2020, 12, 1)
+    end = dt.date(2020, 12, 2)
     counter = 0
 
     def fetch0():
@@ -219,7 +218,7 @@ def test_get_df_with_retries():
         return pd.DataFrame()
 
     def mock_apply(self, *_args, **_kwargs):
-        return self.base_date - datetime.timedelta(days=1)
+        return self.base_date - dt.timedelta(days=1)
 
     with Replace('gs_quant.timeseries.helper.RelativeDate.apply_rule', mock_apply):
         df = get_df_with_retries(fetch0, start, end, 'NYSE')
@@ -233,16 +232,16 @@ def test_get_df_with_retries():
 
 
 def test_forward_looking():
-    today = datetime.date.today()
+    today = dt.date.today()
     source = 'plottool'
-    with DataContext(today, today + datetime.timedelta(days=1)):
+    with DataContext(today, today + dt.timedelta(days=1)):
         assert check_forward_looking('1b', None) is None
         assert check_forward_looking(today, None) is None
         assert check_forward_looking(None, None) is None
         assert check_forward_looking('1b', source) is None
         assert check_forward_looking(today, source) is None
         assert check_forward_looking(None, source) is None
-    with DataContext(today - datetime.timedelta(days=1), today):
+    with DataContext(today - dt.timedelta(days=1), today):
         assert check_forward_looking('1b', None) is None
         assert check_forward_looking(today, None) is None
         assert check_forward_looking(None, None) is None
@@ -257,24 +256,14 @@ def test_get_dataset_data_with_retries():
     GsSession.current = MagicMock(return_calue=NullContextManager())
     mock = replace('gs_quant.data.dataset.Dataset.get_data', Mock())
     mock.side_effect = [
-        MqRequestError(400, message='Number of rows returned by your query is more than maximum allowed'),
-        pd.DataFrame(),
-        pd.DataFrame()
-    ]
-
-    dataset = Dataset(Dataset.TR.TREOD)
-    data = get_dataset_data_with_retries(dataset, start=datetime.date(2000, 1, 2), end=datetime.date(2019, 1, 9),
-                                         assetId='MA4B66MW5E27U8P32SB')
-
-    assert_frame_equal(data, pd.DataFrame())
-
-    mock.side_effect = [
+        MqRequestError(400, message='Some other error'),
+        MqRequestError(400, message='Some other error'),
         MqRequestError(400, message='Some other error')
     ]
-
+    dataset = Dataset(Dataset.TR.TREOD)
     with pytest.raises(MqRequestError):
-        get_dataset_data_with_retries(dataset, start=datetime.date(2000, 1, 2), end=datetime.date(2019, 1, 9),
-                                      assetId='MA4B66MW5E27U8P32SB')
+        get_dataset_data_with_retries(dataset, start=dt.date(2000, 1, 2), end=dt.date(2019, 1, 9),
+                                      assetId='MA4B66MW5E27U8P32SB', max_retries=0)
 
     replace.restore()
 
@@ -308,3 +297,22 @@ def test_split_where_conditions():
 
 if __name__ == "__main__":
     pytest.main(args=["test_helper.py"])
+
+
+def test_get_dataset_data_with_retries_recursive_split():
+    dataset = Mock()
+    dataset.get_data = Mock(side_effect=[
+        MqRequestError(400, message="Some error occurred"),
+        pd.DataFrame({"data": [1, 2]}),
+        pd.DataFrame({"data": [3, 4]})
+    ])
+
+    start = dt.date(2023, 1, 1)
+    end = dt.date(2023, 1, 10)
+
+    result = get_dataset_data_with_retries(dataset, start=start, end=end, max_retries=2)
+
+    # Verify the recursive splitting logic
+    assert not result.empty
+    assert len(result) == 4
+    dataset.get_data.assert_called()
