@@ -13,7 +13,7 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
-import datetime
+import datetime as dt
 from math import sqrt
 
 import pandas as pd
@@ -71,6 +71,27 @@ mock_risk_model_data = {
     ]
 }
 
+mock_risk_model_factor_data_intraday = {
+    'totalResults': 2,
+    'missingDates': [],
+    'results': [
+        {
+            'time': '2020-01-01T10:10:10Z',
+            'factor': 'Factor Name',
+            'factorCategory': 'Style',
+            "factorId": "factor_id",
+            "factorReturn": 1.022
+        },
+        {
+            'time': '2020-01-01T10:10:20Z',
+            'factor': 'Factor Name 1',
+            'factorCategory': 'Style',
+            "factorId": "factor_id_1",
+            "factorReturn": 1.033
+        }
+    ]
+}
+
 mock_risk_model_factor_data = [{
     'identifier': 'factor_id',
     'type': 'Factor',
@@ -83,6 +104,18 @@ mock_covariance_curve = {
     '2020-01-03': 1.03
 }
 
+mock_volatility_curve = {
+    '2020-01-01': sqrt(1.01),
+    '2020-01-02': sqrt(1.02),
+    '2020-01-03': sqrt(1.03)
+}
+
+mock_correlation_curve = {
+    '2020-01-01': 1.0,
+    '2020-01-02': 1.0,
+    '2020-01-03': 1.0
+}
+
 
 def mock_risk_model():
     replace = Replacer()
@@ -96,6 +129,51 @@ def mock_risk_model():
     return actual
 
 
+def test_risk_model_measure():
+    replace = Replacer()
+
+    # mock getting risk model entity()
+    mock = replace('gs_quant.api.gs.risk_models.GsRiskModelApi.get_risk_model', Mock())
+    mock.return_value = mock_risk_model_obj
+
+    # mock getting risk model factor entity
+    mock = replace('gs_quant.api.gs.risk_models.GsFactorRiskModelApi.get_risk_model_data', Mock())
+    mock.return_value = mock_risk_model_data
+
+    # mock getting risk model factor entity
+    mock = replace('gs_quant.api.gs.risk_models.GsFactorRiskModelApi.get_risk_model_factor_data', Mock())
+    mock.return_value = mock_risk_model_factor_data
+
+    # mock getting asset gsid
+    mock = replace('gs_quant.markets.securities.Asset.get_identifier', Mock())
+    mock.return_value = '14593'
+
+    # mock getting risk model dates
+    mock = replace('gs_quant.api.gs.risk_models.GsRiskModelApi.get_risk_model_dates', Mock())
+    mock.return_value = ['2020-01-01', '2020-01-02', '2020-01-03']
+
+    # mock getting risk model data
+    mock = replace('gs_quant.models.risk_model.MarqueeRiskModel.get_data', Mock())
+    mock.return_value = {
+        'totalResults': 3,
+        'missingDates': [],
+        'results': [
+            {'date': '2024-08-19', 'assetData': {'universe': ['14593'], 'bidAskSpread30d': [0.1]}},
+            {'date': '2024-08-20', 'assetData': {'universe': ['14593'], 'bidAskSpread30d': [0.2]}},
+            {'date': '2024-08-21', 'assetData': {'universe': ['14593'], 'bidAskSpread30d': [0.3]}}
+        ]
+    }
+
+    with DataContext(dt.date(2020, 1, 1), dt.date(2020, 1, 3)):
+        actual = mrm.risk_model_measure(Stock(id_='id', name='Fake Asset'), 'model_id',
+                                        mrm.ModelMeasureString.BID_AKS_SPREAD_30D)
+        assert all(actual.values == [0.1, 0.2, 0.3])
+
+    with pytest.raises(AttributeError):
+        mrm.risk_model_measure(Stock(id_='id', name='Fake Asset'), 'model_id', 'Wrong Factor Name')
+    replace.restore()
+
+
 def test_factor_zscore():
     replace = Replacer()
 
@@ -159,7 +237,7 @@ def test_factor_zscore():
         ]
     }
 
-    with DataContext(datetime.date(2020, 1, 1), datetime.date(2020, 1, 3)):
+    with DataContext(dt.date(2020, 1, 1), dt.date(2020, 1, 3)):
         actual = mrm.factor_zscore(Stock(id_='id', name='Fake Asset'), 'model_id', 'Factor Name')
         assert all(actual.values == [1.01, 1.02, 1.03])
 
@@ -191,7 +269,7 @@ def test_factor_covariance():
     mock = replace('gs_quant.markets.factor.Factor.covariance', Mock())
     mock.return_value = mock_covariance_curve
 
-    with DataContext(datetime.date(2020, 1, 1), datetime.date(2020, 1, 3)):
+    with DataContext(dt.date(2020, 1, 1), dt.date(2020, 1, 3)):
         actual = mrm.factor_covariance(mock_risk_model(), 'Factor Name', 'Factor Name')
         assert all(actual.values == [1.01, 1.02, 1.03])
 
@@ -220,10 +298,10 @@ def test_factor_volatility():
     mock.return_value = ['2020-01-01', '2020-01-02', '2020-01-03']
 
     # mock getting covariances
-    mock = replace('gs_quant.markets.factor.Factor.variance', Mock())
-    mock.return_value = mock_covariance_curve
+    mock = replace('gs_quant.markets.factor.Factor.volatility', Mock())
+    mock.return_value = mock_volatility_curve
 
-    with DataContext(datetime.date(2020, 1, 1), datetime.date(2020, 1, 3)):
+    with DataContext(dt.date(2020, 1, 1), dt.date(2020, 1, 3)):
         actual = mrm.factor_volatility(mock_risk_model(), 'Factor Name')
         assert all(actual.values == [sqrt(1.01) * 100, sqrt(1.02) * 100, sqrt(1.03) * 100])
 
@@ -252,12 +330,12 @@ def test_factor_correlation():
     mock.return_value = ['2020-01-01', '2020-01-02', '2020-01-03']
 
     # mock getting covariances
-    mock = replace('gs_quant.markets.factor.Factor.covariance', Mock())
-    mock.return_value = mock_covariance_curve
+    mock = replace('gs_quant.markets.factor.Factor.correlation', Mock())
+    mock.return_value = mock_correlation_curve
 
-    with DataContext(datetime.date(2020, 1, 1), datetime.date(2020, 1, 3)):
+    with DataContext(dt.date(2020, 1, 1), dt.date(2020, 1, 3)):
         actual = mrm.factor_correlation(mock_risk_model(), 'Factor Name', 'Factor Name')
-        assert all(actual.values == [1.0000000000000002, 1, 1])
+        assert all(actual.values == [1, 1, 1])
     replace.restore()
 
 
@@ -284,11 +362,33 @@ def test_factor_performance():
     mock = replace('gs_quant.markets.factor.Factor.returns', Mock())
     mock.return_value = pd.DataFrame.from_dict(mock_covariance_curve, orient='index', columns=['return'])
 
-    with DataContext(datetime.date(2020, 1, 1), datetime.date(2020, 1, 3)):
+    with DataContext(dt.date(2020, 1, 1), dt.date(2020, 1, 3)):
         actual = mrm.factor_performance(mock_risk_model(), 'Factor Name')
         assert len(actual.values) == 3
     replace.restore()
 
 
+def test_factor_performance_intraday():
+    replace = Replacer()
+
+    # mock getting risk model factor entity
+    mock = replace('gs_quant.api.gs.risk_models.GsFactorRiskModelApi.get_risk_model_factor_data', Mock())
+    mock.return_value = mock_risk_model_factor_data
+
+    # mock getting risk model entity()
+    mock = replace('gs_quant.api.gs.risk_models.GsRiskModelApi.get_risk_model', Mock())
+    mock.return_value = mock_risk_model_obj
+
+    # mock getting factor returns
+    mock = replace('gs_quant.markets.factor.Factor.intraday_returns', Mock())
+    mock.return_value = (pd.DataFrame(mock_risk_model_factor_data_intraday.get('results')).set_index('time')
+                         .drop(columns=["factorCategory", "factor", "factorId"], errors='ignore'))
+
+    with DataContext(dt.datetime(2025, 1, 1, 0, 0, 0), dt.datetime(2025, 1, 1, 23, 59, 59)):
+        actual = mrm.factor_performance_intraday(mock_risk_model(), 'Factor Name')
+        assert len(actual.values) == 2
+    replace.restore()
+
+
 if __name__ == '__main__':
     pytest.main(args=[__file__])
