@@ -37,13 +37,21 @@ ENABLE_DISPLAY_NAME = 'GSQ_ENABLE_MEASURE_DISPLAY_NAME'
 USE_DISPLAY_NAME = os.environ.get(ENABLE_DISPLAY_NAME) == "1"
 _logger = logging.getLogger(__name__)
 
+
+class Entitlement(Enum):
+    INTERNAL = 'internal'
+
+
 try:
     from quant_extensions.timeseries.rolling import rolling_apply
 except ImportError as e:
     _logger.debug('unable to import rolling_apply extension: %s', e)
 
     def rolling_apply(s: pd.Series, offset: pd.DateOffset, function: Callable[[np.ndarray], float]) -> pd.Series:
-        values = [function(s.loc[(s.index > idx - offset) & (s.index <= idx)]) for idx in s.index]
+        if isinstance(s.index, pd.DatetimeIndex):
+            values = [function(s.loc[(s.index > (idx - offset)) & (s.index <= idx)]) for idx in s.index]
+        else:
+            values = [function(s.loc[(s.index > (idx - offset).date()) & (s.index <= idx)]) for idx in s.index]
         return pd.Series(values, index=s.index, dtype=np.double)
 
 
@@ -145,7 +153,10 @@ def apply_ramp(x: pd.Series, window: Window) -> pd.Series:
     if isinstance(window.w, int) and window.w > len(x):  # does not restrict window size when it is a DataOffset
         return pd.Series(dtype=float)
     if isinstance(window.r, pd.DateOffset):
-        return x.loc[x.index[0] + window.r:]
+        if np.issubdtype(x.index, dt.date):
+            return x.loc[(x.index[0] + window.r).date():]
+        else:
+            return x.loc[(x.index[0] + window.r).to_pydatetime():]
     else:
         return x[window.r:]
 
@@ -197,7 +208,7 @@ def check_forward_looking(pricing_date, source, name="function"):
 
 def plot_measure(asset_class: tuple, asset_type: Optional[tuple] = None,
                  dependencies: Optional[List[QueryType]] = tuple(), asset_type_excluded: Optional[tuple] = None,
-                 display_name: Optional[str] = None):
+                 display_name: Optional[str] = None, entitlements: Optional[List[Entitlement]] = []):
     # Indicates that fn should be exported to plottool as a member function / pseudo-measure.
     # Set category to None for no restrictions, else provide a tuple of allowed values.
     def decorator(fn):
@@ -213,6 +224,7 @@ def plot_measure(asset_class: tuple, asset_type: Optional[tuple] = None,
         fn.asset_type = asset_type
         fn.asset_type_excluded = asset_type_excluded
         fn.dependencies = dependencies
+        fn.entitlements = entitlements
 
         if USE_DISPLAY_NAME:
             fn.display_name = display_name
@@ -312,7 +324,7 @@ def get_dataset_data_with_retries(dataset: Dataset,
     try:
         data = dataset.get_data(start=start, end=end, **kwargs)
     except MqRequestError as e:
-        if 'Number of rows returned by your query is more than maximum allowed' in e.message and count < max_retries:
+        if count < max_retries:
             mid = start + (end - start) / 2
             count += 1
             first_half = partial(get_dataset_data_with_retries, dataset, start=start, end=mid, count=count, **kwargs)
@@ -367,8 +379,8 @@ def rolling_offset(s: pd.Series,
     """
     # frequencies that can be passed to Series.rolling
     fixed = {
-        'hour': 'H',
-        'hours': 'H',
+        'hour': 'h',
+        'hours': 'h',
         'day': 'D',
         'days': 'D'
     }
