@@ -15,21 +15,25 @@ under the License.
 """
 import datetime as dt
 from enum import Enum
-from typing import Optional
+from typing import Optional, Union, List
 
 import pandas as pd
+import numpy as np
+import math
 from pandas.tseries.offsets import BDay
 from pydash import decapitalize
 
-from gs_quant.api.gs.data import QueryType
+from gs_quant.api.gs.data import QueryType, GsDataApi, DataQuery
 from gs_quant.data.core import DataContext
 from gs_quant.entities.entity import EntityType
 from gs_quant.errors import MqValueError
 from gs_quant.markets.portfolio_manager import PortfolioManager
-from gs_quant.markets.report import FactorRiskReport, PerformanceReport, ThematicReport, ReturnFormat
+from gs_quant.markets.report import FactorRiskReport, PerformanceReport, ThematicReport, ReturnFormat, \
+    format_aum_for_return_calculation, get_pnl_percent, get_factor_pnl_percent_for_single_factor
 from gs_quant.models.risk_model import FactorRiskModel
 from gs_quant.target.reports import PositionSourceType
 from gs_quant.timeseries import plot_measure_entity
+from gs_quant.timeseries.algebra import geometrically_aggregate
 from gs_quant.timeseries.measures import _extract_series_from_df, SecurityMaster, AssetIdentifier
 
 
@@ -252,7 +256,7 @@ def thematic_exposure(report_id: str, basket_ticker: str, *, source: str = None,
                                                end_date=DataContext.current.end_date,
                                                basket_ids=[asset.get_marquee_id()])
     if not df.empty:
-        df.set_index('date', inplace=True)
+        df = df.set_index('date')
         df.index = pd.to_datetime(df.index)
     return _extract_series_from_df(df, QueryType.THEMATIC_EXPOSURE)
 
@@ -276,11 +280,147 @@ def thematic_beta(report_id: str, basket_ticker: str, *, source: str = None,
                                             end_date=DataContext.current.end_date,
                                             basket_ids=[asset.get_marquee_id()])
     if not df.empty:
-        df.set_index('date', inplace=True)
+        df = df.set_index('date')
         df.index = pd.to_datetime(df.index)
     return _extract_series_from_df(df, QueryType.THEMATIC_BETA)
 
 
+@plot_measure_entity(EntityType.REPORT, [QueryType.AUM])
+def aum(report_id: str, *, source: str = None,
+        real_time: bool = False, request_id: Optional[str] = None) -> pd.Series:
+    """
+    AUM of the portfolio
+
+    :param report_id: id of performance report
+    :param source: name of function caller
+    :param real_time: whether to retrieve intraday data instead of EOD
+    :param request_id: server request id
+    :return: portfolio long pnl
+    """
+    start_date = DataContext.current.start_time.date()
+    end_date = DataContext.current.end_time.date()
+    performance_report = PerformanceReport.get(report_id)
+    aum_curve = performance_report.get_aum(start_date=start_date, end_date=end_date)
+    aum_dict = [{'date': key, 'aum': aum_curve[key]} for key in aum_curve]
+
+    # Create and return timeseries
+    df = pd.DataFrame(aum_dict)
+    if not df.empty:
+        df = df.set_index('date')
+        df.index = pd.to_datetime(df.index)
+    return _extract_series_from_df(df, QueryType.AUM)
+
+
+@plot_measure_entity(EntityType.REPORT, [QueryType.PNL])
+def pnl(report_id: str, unit: str = 'Notional', *, source: str = None,
+        real_time: bool = False, request_id: Optional[str] = None) -> pd.Series:
+    """
+    PNL from all holdings, if unit is Percent, geometrically aggregate over time frame
+
+    :param report_id: id of performance report
+    :param unit: by default uses mode as Notional, but can also be set to Percent
+    :param source: name of function caller
+    :param real_time: whether to retrieve intraday data instead of EOD
+    :param request_id: server request id
+    :return: portfolio pnl
+    """
+    start_date = DataContext.current.start_time.date()
+    end_date = DataContext.current.end_time.date()
+    performance_report = PerformanceReport.get(report_id)
+    pnl_df = performance_report.get_pnl(start_date, end_date)
+
+    if unit == Unit.PERCENT.value:
+        return get_pnl_percent(performance_report, pnl_df, 'pnl', start_date, end_date)
+    else:
+        pnl_df = pnl_df.set_index('date')
+        return pd.Series(pnl_df['pnl'], name="pnl")
+
+
+@plot_measure_entity(EntityType.REPORT)
+def historical_simulation_estimated_pnl(report_id: str, *, source: str = None,
+                                        real_time: bool = False, request_id: Optional[str] = None) -> pd.Series:
+    """
+    Estimated Pnl from replaying a historical simulation scenario on your latest positions
+    :param report_id: id of performance report
+    :param source: name of function caller
+    :param real_time: whether to retrieve intraday data instead of EOD
+    :param request_id: server request id
+    :return: portfolio estimated pnl
+    """
+
+    factor_attributed_pnl = _replay_historical_factor_moves_on_latest_positions(report_id, [])
+    total_factor_attributed_pnl = factor_attributed_pnl.apply(np.sum, axis=1).to_frame("estimatedPnl")
+
+    total_factor_attributed_pnl.index = pd.to_datetime(total_factor_attributed_pnl.index)
+    return total_factor_attributed_pnl.squeeze()
+
+
+@plot_measure_entity(EntityType.REPORT)
+def historical_simulation_estimated_factor_attribution(report_id: str, factor_name: str, *, source: str = None,
+                                                       real_time: bool = False,
+                                                       request_id: Optional[str] = None) -> pd.Series:
+    """
+    Estimated Pnl attributed to the factor after replaying a historical simulation scenario on a portfolio's latest
+    positions
+    :param report_id: id of performance report
+    :param factor_name: name of the factor
+    :param source: name of function caller
+    :param real_time: whether to retrieve intraday data instead of EOD
+    :param request_id: server request id
+    :return: portfolio estimated pnl
+    """
+
+    factor_attributed_pnl = _replay_historical_factor_moves_on_latest_positions(report_id, [factor_name])
+    factor_attributed_pnl.index = pd.to_datetime(factor_attributed_pnl.index)
+
+    return factor_attributed_pnl.squeeze()
+
+
+def _replay_historical_factor_moves_on_latest_positions(report_id: str, factors: List[str]) -> \
+        Union[pd.Series, pd.DataFrame]:
+    start_date = DataContext.current.start_time.date()
+    end_date = DataContext.current.end_time.date()
+    risk_report = FactorRiskReport.get(report_id)
+    risk_model_id = risk_report.get_risk_model_id()
+
+    # Get data in batches of 365 days
+    date_range = pd.bdate_range(start_date, end_date)
+    batches = np.array_split([d.date() for d in date_range.tolist()], math.ceil(len(date_range.tolist()) / 365))
+    data_query_results = []
+    query = {"riskModel": risk_model_id}
+    if factors:
+        query.update({"factor": factors})
+    for batch in batches:
+        data_query_results += GsDataApi.execute_query(
+            'RISK_MODEL_FACTOR',
+            DataQuery(
+                where=query,
+                start_date=batch[0],
+                end_date=batch[-1]
+            )
+        ).get('data', [])
+
+    return_data = pd.DataFrame(data_query_results).pivot(columns="factor", index="date", values="return").sort_index()
+    return_data_aggregated = (return_data / 100).apply(geometrically_aggregate)
+
+    latest_report_date = risk_report.latest_end_date
+    factor_exposures = risk_report.get_results(
+        start_date=latest_report_date,
+        end_date=latest_report_date,
+        return_format=ReturnFormat.JSON
+    )
+    factor_exposure_df = pd.DataFrame(factor_exposures).pivot(columns="factor",
+                                                              index="date",
+                                                              values="exposure").sort_index()
+
+    factor_exposure_df = factor_exposure_df.reindex(columns=return_data_aggregated.columns)
+    factor_attributed_pnl_values = return_data_aggregated.values * factor_exposure_df.values
+    factor_attributed_pnl = pd.DataFrame(factor_attributed_pnl_values, index=return_data_aggregated.index,
+                                         columns=return_data_aggregated.columns)
+
+    return factor_attributed_pnl
+
+
 def _get_factor_data(report_id: str, factor_name: str, query_type: QueryType, unit: Unit = Unit.NOTIONAL) -> pd.Series:
     # Check params
     report = FactorRiskReport.get(report_id)
@@ -295,29 +435,30 @@ def _get_factor_data(report_id: str, factor_name: str, query_type: QueryType, un
     col_name = query_type.value.replace(' ', '')
     col_name = decapitalize(col_name)
     data_type = decapitalize(col_name[6:]) if col_name.startswith('factor') else col_name
-
+    include_total_factor = factor_name != 'Total' and unit == Unit.PERCENT and query_type == QueryType.FACTOR_PNL
+    factors_to_query = [factor_name, 'Total'] if include_total_factor else [factor_name]
     factor_data = report.get_results(
-        factors=[factor_name],
+        factors=factors_to_query,
         start_date=DataContext.current.start_date,
         end_date=DataContext.current.end_date,
         return_format=ReturnFormat.JSON
     )
-    factor_data = [d for d in factor_data if d.get(data_type)]
+    total_data = [d for d in factor_data if d.get(data_type) is not None and d.get('factor') == 'Total']
+    factor_data = [d for d in factor_data if d.get(data_type) is not None and d.get('factor') == factor_name]
     if unit == Unit.PERCENT:
         if report.position_source_type != PositionSourceType.Portfolio:
             raise MqValueError('Unit can only be set to percent for portfolio reports')
         pm = PortfolioManager(report.position_source_id)
+        tags = dict((tag.name, tag.value) for tag in report.parameters.tags) if report.parameters.tags else None
+        performance_report = pm.get_performance_report(tags=tags)
+        start_date = dt.datetime.strptime(min([d['date'] for d in factor_data]), '%Y-%m-%d').date()
+        end_date = dt.datetime.strptime(max([d['date'] for d in factor_data]), '%Y-%m-%d').date()
         if query_type == QueryType.FACTOR_PNL:
-            last_date = dt.datetime.strptime(max([d['date'] for d in factor_data]), '%Y-%m-%d').date()
-            aum = pm.get_aum(start_date=last_date, end_date=last_date)
-            last_aum = aum.get(last_date.strftime('%Y-%m-%d'))
-            if last_aum is None:
-                raise MqValueError('Cannot convert to percent: Missing AUM on the last date in the date range')
-            factor_exposures = [{'date': d['date'], col_name: d[data_type] / last_aum * 100} for d in factor_data]
+            # Factor Pnl needs to be geometrically aggregated if unit is %
+            aum_df = format_aum_for_return_calculation(performance_report, start_date, end_date)
+            return get_factor_pnl_percent_for_single_factor(factor_data, total_data, aum_df, start_date)
         else:
-            start_date = dt.datetime.strptime(min([d['date'] for d in factor_data]), '%Y-%m-%d').date()
-            end_date = dt.datetime.strptime(max([d['date'] for d in factor_data]), '%Y-%m-%d').date()
-            aum = pm.get_aum(start_date=start_date, end_date=end_date)
+            aum = performance_report.get_aum(start_date=start_date, end_date=end_date)
             for data in factor_data:
                 if aum.get(data['date']) is None:
                     raise MqValueError('Cannot convert to percent: Missing AUM on some dates in the date range')
@@ -331,7 +472,7 @@ def _get_factor_data(report_id: str, factor_name: str, query_type: QueryType, un
     # Create and return timeseries
     df = pd.DataFrame(factor_exposures)
     if not df.empty:
-        df.set_index('date', inplace=True)
+        df = df.set_index('date')
         df.index = pd.to_datetime(df.index)
     return _extract_series_from_df(df, query_type)
 
@@ -344,7 +485,7 @@ def _return_metrics(one_leg: pd.DataFrame, dates: list, name: str):
     one_leg['cumulativePnl'] = one_leg['pnl'].cumsum(axis=0)
 
     one_leg['normalizedExposure'] = (one_leg['exposure'] - one_leg['cumulativePnl'])
-    one_leg['cumulativePnl'].iloc[0] = 0
+    one_leg.iloc[0, one_leg.columns.get_loc('cumulativePnl')] = 0
     one_leg[f'{name}Metrics'] = one_leg['cumulativePnl'] / one_leg['normalizedExposure'] + 1
 
     one_leg[f'{name}Metrics'] = 1 / one_leg[f'{name}Metrics'] if one_leg['exposure'].iloc[-1] < 0 else one_leg[
